@@ -34,7 +34,7 @@
  *
  */
                                                                                                           
-/* $Id: palette.c,v 1.12 2003/03/14 02:25:30 famao Exp $ */
+/* $Id: palette.c,v 1.3.2.17 2003/01/22 15:24:32 famao Exp $ */
                                                                                                           
 /* vi:set ts=4 sw=4: */
 
@@ -65,11 +65,11 @@ _configure_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer user_data)
 		}
 #endif
 		palette->running_wm = wmutil_window_check_wm_type ();
+		g_message ("running wm %d", palette->running_wm);
 		if (palette->running_wm) {
 			gint new_wm_type = (palette->running_wm ^ old_wm);
+			g_message ("new_wm_type %d", new_wm_type);
 			if (new_wm_type) {
-				g_message ("IIIMF-SKK detected WM %d", palette->running_wm);
-				g_message ("IIIMF-SKK Append WM Type %d", new_wm_type);
 				palette->detected_wm (palette, palette->running_wm, new_wm_type);
 				if (palette->running_wm & WM_CAN_TRAY)
 					palette->detected_tray (palette, palette->running_wm, new_wm_type);
