@@ -34,7 +34,7 @@
  *
  */
                                                                                                           
-/* $Id: wmutil.c,v 1.10 2004/04/25 02:51:44 famao Exp $ */
+/* $Id: wmutil.c,v 1.2.2.8 2003/03/27 22:18:25 famao Exp $ */
                                                                                                           
 /* vi:set ts=4 sw=4: */
 
@@ -82,9 +82,11 @@ _wmutil_window_tray_get_owner (GdkWindow *window)
 			buffer , TRUE);
 	if (tray == None)
 		return None;
-	owner = XGetSelectionOwner (GDK_WINDOW_XDISPLAY (window), tray);
+	owner = XGetSelectionOwner (GDK_WINDOW_XDISPLAY (window),
+			tray);
 	if (!owner)
 		return None;
+	g_message ("owner found %lx", owner);
 	return owner;
 }
 
@@ -229,10 +231,7 @@ wmutil_window_state_ewmh_prop (GdkWindow *state, GdkAtom *atoms, guint num, GdkP
 void
 wmutil_window_current_desktop_ewmh (GdkWindow *window, gint *desktop)
 {
-	union {
-		long *lval;
-		guchar *uval;
-	}value;
+	long *value;
 	GdkAtom type_r, cur_desktop;
 	gint format_r, length_r;
 	cur_desktop = gdk_atom_intern ("_NET_CURRENT_DESKTOP", TRUE);
@@ -247,10 +246,10 @@ wmutil_window_current_desktop_ewmh (GdkWindow *window, gint *desktop)
 				&type_r,
 				&format_r,
 				&length_r,
-				&value.uval)) {
+				(guchar **)&value)) {
 		if (desktop)
-			*desktop = value.lval[0];
-		g_free (value.uval);
+			*desktop = value[0];
+		g_free (value);
 		return;
 	}
 
@@ -261,13 +260,10 @@ error:
 void
 wmutil_window_workarea_ewmh (GdkWindow *window, gint *x, gint *y, gint *width, gint *height)
 {
+	long *value;
 	GdkAtom type_r, workarea;
 	gint format_r, length_r;
 	gint cur_desktop;
-	union {
-		long *lval;
-		guchar *uval;
-	}value;
 
 	wmutil_window_current_desktop_ewmh (window, &cur_desktop);
 	if (cur_desktop < 0)
@@ -285,12 +281,12 @@ wmutil_window_workarea_ewmh (GdkWindow *window, gint *x, gint *y, gint *width, g
 				&type_r,
 				&format_r,
 				&length_r,
-				&value.uval)) {
-		if (x) *x = value.lval[cur_desktop * 4];
-		if (y) *y = value.lval[cur_desktop * 4 + 1];
-		if (width) *width = value.lval[cur_desktop * 4 + 2];
-		if (height) *height = value.lval[cur_desktop * 4 + 3];
-		g_free (value.uval);
+				(guchar **)&value)) {
+		if (x) *x = value[cur_desktop * 4];
+		if (y) *y = value[cur_desktop * 4 + 1];
+		if (width) *width = value[cur_desktop * 4 + 2];
+		if (height) *height = value[cur_desktop * 4 + 3];
+		g_free (value);
 		return;
 	}
 	
@@ -305,12 +301,9 @@ error:
 static gboolean
 is_windowmaker_wm (void)
 {
+	long *value;
 	GdkAtom type_r, iswm;
 	gint format_r, length_r;
-	union {
-		long *lval;
-		guchar *uval;
-	}value;
 	iswm = gdk_atom_intern ("_WINDOWMAKER_WM_PROTOCOLS", TRUE);
 	if (iswm == GDK_NONE)
 		return FALSE;
@@ -323,8 +316,8 @@ is_windowmaker_wm (void)
 				&type_r,
 				&format_r,
 				&length_r,
-				&value.uval)) {
-		g_free (value.uval);
+				(guchar **)&value)) {
+		g_free (value);
 		if (length_r > 0 && XA_ATOM == ATOM_TO_XATOM (GDK_ROOT_PARENT(), type_r) ) {
 			return TRUE;
 		} else {
@@ -347,12 +340,9 @@ is_icewm_wm (void)
 static gboolean
 is_blackbox_wm (void)
 {
+	long *value;
 	GdkAtom type_r, isbb;
 	gint format_r, length_r;
-	union {
-		long *lval;
-		guchar *uval;
-	}value;
 	/*
 	 * blackbox-based window manager sets _BLACKBOX_PID property
 	 * to RootWindow
@@ -369,8 +359,8 @@ is_blackbox_wm (void)
 				&type_r,
 				&format_r,
 				&length_r,
-				&value.uval)) {
-		g_free (value.uval);
+				(guchar **)&value)) {
+		g_free (value);
 		if (length_r > 0 && type_r != None) {
 			return TRUE;
 		} else {
@@ -383,13 +373,9 @@ is_blackbox_wm (void)
 static gboolean
 is_kde_wm (void)
 {
+	long *value;
 	GdkAtom type_r, iskde;
 	gint format_r, length_r;
-	union {
-		long *lval;
-		guchar *uval;
-	}value;
-
 	iskde = gdk_atom_intern ("KWIN_RUNNING", TRUE);
 	if (iskde == GDK_NONE)
 		return FALSE;
@@ -402,8 +388,8 @@ is_kde_wm (void)
 				&type_r,
 				&format_r,
 				&length_r,
-				&value.uval)) {
-		g_free (value.uval);
+				(guchar **)&value)) {
+		g_free (value);
 		if (length_r > 0 && type_r == iskde) {
 			return TRUE;
 		} else {
@@ -416,13 +402,9 @@ is_kde_wm (void)
 static gboolean
 is_gnome_wm (void)
 {
+	long *value;
 	GdkAtom type_r, isgnome;
 	gint format_r, length_r;
-	union {
-		long *lval;
-		guchar *uval;
-	}value;
-
 	isgnome = gdk_atom_intern ("_WIN_SUPPORTING_WM_CHECK", TRUE);
 	if (isgnome == GDK_NONE)
 		return FALSE;
@@ -435,8 +417,8 @@ is_gnome_wm (void)
 				&type_r,
 				&format_r,
 				&length_r,
-				&value.uval)) {
-		g_free (value.uval);
+				(guchar **)&value)) {
+		g_free (value);
 		if (length_r > 0 &&
 				ATOM_TO_XATOM (GDK_ROOT_PARENT(), type_r) == XA_CARDINAL) {
 			return TRUE;
@@ -450,13 +432,9 @@ is_gnome_wm (void)
 static gboolean
 is_ewmh_wm (void)
 {
+	long *value;
 	GdkAtom type_r, isewmh;
 	gint format_r, length_r;
-	union {
-		long *lval;
-		guchar *uval;
-	}value;
-
 	isewmh = gdk_atom_intern ("_NET_SUPPORTED", TRUE);
 	if (isewmh == GDK_NONE)
 		return FALSE;
@@ -469,8 +447,8 @@ is_ewmh_wm (void)
 				&type_r,
 				&format_r,
 				&length_r,
-				&value.uval)) {
-		g_free (value.uval);
+				(guchar **)&value)) {
+		g_free (value);
 		if (length_r > 0 &&
 				ATOM_TO_XATOM (GDK_ROOT_PARENT(), type_r) == XA_ATOM) {
 			return TRUE;
@@ -487,11 +465,13 @@ can_trayed_spec (void)
 	Window owner;
 	GdkAtom tray = gdk_atom_intern ("_NET_SYSTEM_TRAY_S0", TRUE);
 	if (tray == GDK_NONE) {
+		g_message ("tray atom not found");
 		return FALSE;
 	}
 	owner = XGetSelectionOwner (GDK_DISPLAY (), 
 			ATOM_TO_XATOM(GDK_ROOT_PARENT(),tray));
 	if (owner) {
+		g_message ("owner found");
 		return TRUE;
 	}
 	return FALSE;
@@ -535,6 +515,7 @@ wmutil_window_tray_spec (GdkWindow *window)
 	owner = _wmutil_window_tray_get_owner (window);
 	if (owner == None)
 		return;
+	g_message ("owner found %lx", owner);
 	e.type = ClientMessage;
 	e.window = owner;
 	e.message_type = ATOM_TO_XATOM (window, gdk_atom_intern ("_NET_SYSTEM_TRAY_OPCODE", FALSE));
@@ -672,32 +653,3 @@ wmutil_window_hints_icccm (GdkWindow *window, gint state)
 			GDK_WINDOW_XWINDOW (window), &hints);
 	return;
 }
-
-Window
-wmutil_window_get_toplevel (Display *d, Window w)
-{
-	Window ret;
-	Window parent;
-	Window root_win;
-	Window *child_list;
-	guint num_children;
-
-	if (w == None)
-		return None;
-
-	ret = w;
-#if 1
-	while (TRUE) {
-		XQueryTree (d, ret, &root_win, &parent, &child_list, &num_children);
-		XFree (child_list);
-		if (parent == root_win)
-			break;
-		ret = parent;
-	}
-#else
-		XQueryTree (d, ret, &root_win, &parent, &child_list, &num_children);
-		XFree (child_list);
-		ret = parent;
-#endif
-	return ret;
-}
