@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: skkbuffer.c,v 1.28 2004/04/23 07:20:57 famao Exp $ */
+/* $Id: skkbuffer.c,v 1.14.2.48 2003/04/17 06:19:21 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -56,6 +56,7 @@ static gchar* get_buf (SkkBuffer *buf, gint ch, gboolean lower);
 static gchar* get_translate_buf (SkkBuffer *buf);
 static gchar* get_direction_word (SkkBuffer *buf);
 static gchar* create_candidate (SkkBuffer *buf);
+static gboolean get_completion (SkkBuffer *buf);
 
 static gboolean delete_backward_okurigana (SkkBuffer *buf);
 static gboolean delete_backward_buf (SkkBuffer *buf);
@@ -86,6 +87,7 @@ static void set_preedit_buf (SkkBuffer *buf, const gchar *value);
 static void set_buf (SkkBuffer *buf, const gchar *value);
 static void set_direction_word (SkkBuffer *buf, const gchar *value);
 static void set_numb (SkkBuffer *buf, const gchar *value);
+static void set_dcomp (SkkBuffer *buf, const gchar *value);
 static void set_lasth (SkkBuffer *buf);
 
 static void add_okurigana (SkkBuffer *buf, const gchar *append);
@@ -142,6 +144,36 @@ struct _SkkLookupFunc {
 	gpointer user_data;
 };
 
+static gboolean
+get_completion (SkkBuffer *buf)
+{
+	GList *append = NULL;
+	if (!buf)
+		return FALSE;
+	if (buf->completion_list) {
+		skk_utils_list_free (buf->completion_list, TRUE, NULL, NULL);
+		buf->completion_count = 0;
+		buf->completion_list = NULL;
+	}
+	if (!skkbuf_d (buf))
+		return FALSE;
+	/* initialized before completion */
+	skk_query_set_first (buf->query);
+	buf->completion_list = skk_query_do_completion (buf->query, skkbuf_d (buf));
+	while (skk_query_set_next (buf->query)) {
+		append = skk_query_do_completion (buf->query, skkbuf_d (buf));
+		if (append)
+			buf->completion_list = skk_utils_list_merge_string (buf->completion_list, append, TRUE);
+	}
+	/* re-initialized */
+	skk_query_set_first (buf->query);
+	if (!buf->completion_list)
+		return FALSE;
+	/* TODO */
+	/* Translate num? */
+	return TRUE;
+}
+
 static gchar *
 get_direction_word (SkkBuffer *buf)
 {
@@ -177,7 +209,7 @@ move_right (SkkBuffer *buf)
 	if (!skkbuf_p (buf)) {
 		return;
 	}
-	if (skk_buffer_get_query_status (buf) != QUERY_NORMAL) {
+	if (skk_buffer_get_query_mode (buf) != SKK_QUERY_BEGIN) {
 		return;
 	}
 	if (cursor->caret > 0) {
@@ -195,9 +227,15 @@ move_left (SkkBuffer *buf)
 	if (!skkbuf_p (buf)) {
 		return;
 	}
-	if (skk_buffer_get_query_status (buf) != QUERY_NORMAL) {
+	if (skk_buffer_get_query_mode (buf) != SKK_QUERY_BEGIN) {
 		return;
 	}
+#if 0
+	if (skk_conf_get_bool (skkbuf_conf (buf), "skk-dcomp-activate")) {
+		if (skkbuf_dcomp (buf)) {
+		}
+	}
+#endif
 	len = skk_utils_strlen (skkbuf_p (buf));
 	if (cursor->caret < len) {
 		cursor->caret++;
@@ -235,9 +273,15 @@ separate_string_by_cursor (SkkBuffer *buf, const gchar *string, gchar **beforep,
 		after += offset;
 		if (beforep) {
 			*beforep = g_strndup (string, offset);
+#if 0
+			g_message ("beforep %s", *beforep);
+#endif
 		}
 		if (afterp) {
 			*afterp = g_strdup (after);
+#if 0
+			g_message ("after %s", after);
+#endif
 		}
 		return TRUE;
 	}
@@ -338,11 +382,14 @@ preedit_emit (SkkBuffer *buf)
 		g_free (preedit);
 		preedit = NULL;
 	}
-	if (skk_buffer_get_query_status (buf) == QUERY_DONE) {
+	if (skk_buffer_get_query_mode (buf) == SKK_QUERY_DONE) {
 		skkbuf_cursor (buf)->revPos = 2;
 		if (skkbuf_cursor (buf)->caret == 0) {
 			skkbuf_cursor (buf)->revLen = skkbuf_plen (buf);
 		}
+#if 0
+		g_message ("revLen %d", skkbuf_cursor (buf)->revLen);
+#endif
 	} else {
 		skkbuf_cursor (buf)->revPos = 0;
 		skkbuf_cursor (buf)->revLen = 0;
@@ -380,8 +427,8 @@ undo (SkkBuffer *buf)
 {
 	if (!buf)
 		return FALSE;
-	/* If we are not in QUERY_DONE, we cannot undo */
-	if (skk_buffer_get_query_status (buf) != QUERY_DONE)
+	/* If we are not in SKK_QUERY_DONE, we cannot undo */
+	if (skk_buffer_get_query_mode (buf) != SKK_QUERY_DONE)
 		return FALSE;
 	if (skk_buffer_set_prev_candidate (buf))
 		return TRUE;
@@ -396,10 +443,7 @@ undo (SkkBuffer *buf)
 	buf->candidate_count = 0;
 	/* reset query_item */
 	skk_query_set_first (buf->query);
-	skk_buffer_set_query_status (buf, QUERY_NORMAL);
-	if (skk_buffer_get_status (buf) == SKK_J_MODE) {
-		skk_buffer_set_j_status (buf, SKK_HENKAN);
-	}
+	skk_buffer_set_query_mode (buf, SKK_QUERY_BEGIN);
 	preedit_emit (buf);
 	return TRUE;
 }
@@ -409,11 +453,11 @@ change_category (SkkBuffer *buf)
 {
 	/* HIRAGANA <-> KATAKANA */
 	/* LATIN <-> JISX0208_LATIN */
-	SkkStatus status;
+	SkkMajorMode status;
 	gchar *result = NULL;
 	if (!buf)
 		return;
-	status = skk_buffer_get_status (buf);
+	status = skk_buffer_get_mode (buf);
 	switch (status) {
 		case SKK_J_MODE:
 			/* if we don't have preedit, nothing to do */
@@ -458,6 +502,8 @@ change_category (SkkBuffer *buf)
 			result_emit (buf);
 			preedit_emit (buf);
 			break;
+		default:
+			break;
 	}
 	if (result)
 		g_free (result);
@@ -486,7 +532,12 @@ query_num (SkkBuffer *buf)
 		if (type == 4) {
 			/* in the case of #4, retry */
 			/* reset query item */
-			skk_query_set_first (skkbuf_query (buf));
+			if (!skkbuf_retry_query (buf)) {
+				skk_query_set_first (skkbuf_query (buf));
+				SKK_BUFFER_SET_FLAGS(buf, SKK_BUFFER_RETRY_QUERY);
+			} else {
+				skk_query_set_next (buf->query);
+			}
 			/* query to dict */
 			num = skk_query_do_query (buf->query, ret, skkbuf_b (buf), &list_num);
 			if (!num) {
@@ -561,7 +612,7 @@ query_dict (SkkBuffer *buf)
 				}
 			}
 		}
-		candidate_count = skk_conf_get_num_by_name (skkbuf_conf (buf), "skk-candidate-count");
+		candidate_count = skk_conf_get_num (skkbuf_conf (buf), "skk-candidate-count");
 		preedit_emit (buf);
 		if (candidate_count == 0) {
 			skk_buffer_lookup_emit (buf);
@@ -782,6 +833,28 @@ set_buf (SkkBuffer *buf, const gchar *value)
 	return;
 }
 
+static void
+set_dcomp (SkkBuffer *buf, const gchar *value)
+{
+	if (!buf)
+		return;
+	if (value && skkbuf_dcomp (buf)) {
+		if (!strcmp (skkbuf_dcomp (buf), value)) {
+			return;
+		}
+	} else if (!value && !skkbuf_dcomp (buf)) {
+		return;
+	}
+	if (skkbuf_dcomp (buf)) {
+		g_free (skkbuf_dcomp (buf));
+		skkbuf_dcomp (buf) = NULL;
+	}
+	if (value) {
+		skkbuf_dcomp (buf) = g_strdup (value);
+	}
+	return;
+}
+
 static void
 set_numb (SkkBuffer *buf, const gchar *value)
 {
@@ -856,10 +929,10 @@ add_buffer_normal (SkkBuffer *buf, int ch)
 		/* Can not translate to KANA */
 		/* FAMAO */
 		/* */
-		if (buf->check_before) {
+		if (skkbuf_has_many_candidate (buf)) {
 			gchar *tmp_preedit;
 			tmp_preedit = get_translate_buf (buf);
-			buf->check_before = FALSE;
+			SKK_BUFFER_UNSET_FLAGS(buf, SKK_BUFFER_HAS_MANY_CANDIDATE);
 			if (tmp_preedit) {
 				g_free (tmp_in_buf);
 				set_buf (buf, NULL);
@@ -903,7 +976,7 @@ add_buffer_normal (SkkBuffer *buf, int ch)
 		set_buf (buf,tmp_in_buf);
 		if (buf_is_exist > 1) {
 			/* we have more candidates than one */
-			buf->check_before = TRUE;
+			SKK_BUFFER_SET_FLAGS(buf, SKK_BUFFER_HAS_MANY_CANDIDATE);
 			g_free (tmp_in_buf);
 			preedit_emit (buf);
 			return TRUE;
@@ -932,20 +1005,24 @@ add_buffer_henkan (SkkBuffer *buf, int ch)
 	gint buf_is_exist = 0;
 	gchar *tmp_in_buf;
 	gchar *tmp_preedit;
-	DEBUG_DO (printf ("skkbuffer.c : Entering add_buffer_henkan\n"));
 
 	tmp_in_buf = get_buf (buf, ch, TRUE);
 	if (!tmp_in_buf)
 		return FALSE;
-	
+
+	if (skk_conf_get_bool (skkbuf_conf (buf), "skk-dcomp-activate")) {
+		if (skkbuf_dcomp (buf)) {
+			set_preedit_buf (buf, skkbuf_dcomp (buf));
+		}
+	}
 	/* Check whether we can translate buf to KANA */
 	buf_is_exist = skk_conv_is_exist (buf->rule, tmp_in_buf);
 
 	if (!buf_is_exist) {
 		/* FAMAO NOW */
-		if (buf->check_before) {
+		if (skkbuf_has_many_candidate (buf)) {
 			tmp_preedit = get_translate_buf (buf);
-			buf->check_before = FALSE;
+			SKK_BUFFER_UNSET_FLAGS(buf, SKK_BUFFER_HAS_MANY_CANDIDATE);
 			if (tmp_preedit) {
 				g_free (tmp_in_buf);
 				set_buf (buf, NULL);
@@ -976,6 +1053,20 @@ add_buffer_henkan (SkkBuffer *buf, int ch)
 		}
 		/* Change Last Buffer */
 		/* ex) k + p -> p */
+#if 0
+		if (!skkbuf_b (buf)) {
+			skkbuf_b (buf) = tmp_in_buf;
+			skkbuf_blen (buf) = strlen (tmp_in_buf);
+			preedit_emit (buf);
+			return TRUE;
+		}
+#endif
+		if (!skkbuf_b (buf)) {
+			buf->in_buf = tmp_in_buf;
+			skkbuf_blen (buf) = strlen (skkbuf_b (buf));
+			preedit_emit (buf);
+			return TRUE;
+		}
 		buf->in_buf[buf->buf_len -1] = tolower (ch);
 		skkbuf_blen (buf) = strlen (skkbuf_b (buf));
 		g_free (tmp_in_buf);
@@ -985,7 +1076,7 @@ add_buffer_henkan (SkkBuffer *buf, int ch)
 		set_buf (buf,tmp_in_buf);
 		if (buf_is_exist > 1) {
 			/* we have more candidates than one */
-			buf->check_before = TRUE;
+			SKK_BUFFER_SET_FLAGS(buf, SKK_BUFFER_HAS_MANY_CANDIDATE);
 			g_free (tmp_in_buf);
 			preedit_emit (buf);
 			return TRUE;
@@ -994,8 +1085,18 @@ add_buffer_henkan (SkkBuffer *buf, int ch)
 		tmp_preedit = get_translate_buf (buf);
 		if (tmp_preedit) {
 			add_preedit_buf (buf, tmp_preedit);
-			/* FIXME */
 			add_direction_word (buf, tmp_preedit);
+			if (skk_conf_get_bool (skkbuf_conf (buf), "skk-dcomp-activate")) {
+				/* do completion */
+				if (get_completion (buf)) {
+					gchar *value;
+					value = (gchar *)g_list_nth_data (buf->completion_list, 0);
+					if (value && !skkbuf_b (buf)) {
+						set_dcomp (buf, skkbuf_p (buf));
+						set_preedit_buf (buf, value);
+					} 
+				}
+			}
 			g_free (tmp_preedit);
 		}
 		g_free (tmp_in_buf);
@@ -1028,9 +1129,9 @@ add_buffer_okuri (SkkBuffer *buf, int ch)
 			g_free (tmp_in_buf);
 			return FALSE;
 		}
-		if (buf->check_before) {
+		if (skkbuf_has_many_candidate (buf)) {
 			tmp_okurigana = get_translate_buf (buf);
-			buf->check_before = FALSE;
+			SKK_BUFFER_UNSET_FLAGS(buf, SKK_BUFFER_HAS_MANY_CANDIDATE);
 			if (tmp_okurigana) {
 				if (skkbuf_b (buf)) {
 					if (skkbuf_okurip (buf)) {
@@ -1044,8 +1145,7 @@ add_buffer_okuri (SkkBuffer *buf, int ch)
 					preedit_emit (buf);
 					return TRUE;
 				}
-				skk_buffer_set_j_status (buf, SKK_CHOICE);
-				skk_buffer_set_query_status (buf, QUERY_DONE);
+				skk_buffer_set_query_mode (buf, SKK_QUERY_DONE);
 				add_okurigana (buf, tmp_okurigana);
 				if (!skkbuf_okurip (buf))
 					set_okuri_prefix (buf, skkbuf_b (buf));
@@ -1070,7 +1170,7 @@ add_buffer_okuri (SkkBuffer *buf, int ch)
 		set_buf (buf,tmp_in_buf);
 		if (buf_is_exist > 1) {
 			/* we have more candidates than one */
-			buf->check_before = TRUE;
+			SKK_BUFFER_SET_FLAGS(buf, SKK_BUFFER_HAS_MANY_CANDIDATE);
 			g_free (tmp_in_buf);
 			preedit_emit (buf);
 			return TRUE;
@@ -1095,8 +1195,7 @@ add_buffer_okuri (SkkBuffer *buf, int ch)
 			}
 			/* TODO */
 			/* prepare stop_emit */
-			skk_buffer_set_j_status (buf, SKK_CHOICE);
-			skk_buffer_set_query_status (buf, QUERY_DONE);
+			skk_buffer_set_query_mode (buf, SKK_QUERY_DONE);
 			add_okurigana (buf, tmp_okurigana);
 			if (!skkbuf_okurip (buf))
 				set_okuri_prefix (buf, tmp_in_buf);
@@ -1124,7 +1223,7 @@ add_buffer_choice (SkkBuffer *buf, int ch)
 	skk_buffer_commit (buf, TRUE);
 	skk_buffer_clear (buf);
 	if (isupper (ch)) {
-		skk_buffer_set_j_status (buf, SKK_HENKAN);
+		skk_buffer_set_query_mode (buf, SKK_QUERY_BEGIN);
 		return add_buffer_henkan (buf, ch);
 	} else {
 		return add_buffer_normal (buf, ch);
@@ -1135,55 +1234,26 @@ add_buffer_choice (SkkBuffer *buf, int ch)
 static void
 add_mode_check (SkkBuffer *buf, int ch)
 {
-	SkkJStatus jstatus;
-	DEBUG_DO (printf ("skkbuffer.c : Entering add_mode_check\n"));
+	SkkQueryMode qmode;
 	if (!buf)
 		return;
-	jstatus = skk_buffer_get_j_status (buf);
+	qmode = skk_buffer_get_query_mode (buf);
 	if (isupper (ch)) {
-		switch (jstatus) {
-			case SKK_NONE:
-				/* HENKAN */
-				skk_buffer_set_j_status (buf, SKK_HENKAN);
+		switch (qmode) {
+			case SKK_QUERY_NONE:
+				/* begin query */
+				skk_buffer_set_query_mode (buf, SKK_QUERY_BEGIN);
 				break;
-			case SKK_HENKAN:
+			case SKK_QUERY_BEGIN:
 				if (skkbuf_p (buf)) {
-					skk_buffer_set_j_status (buf, SKK_OKURI);
+					skk_buffer_set_query_mode (buf, SKK_QUERY_OKURI);
 					translate_buffer_before_okuri (buf);
 				}
 				break;
-			case SKK_OKURI:
+			case SKK_QUERY_OKURI:
+			case SKK_QUERY_DONE:
 				/* nothing to do */
 				break;
-			case SKK_CHOICE:
-				/* nothing to do */
-				/* HENKAN */
-#if 0
-				buf->store_result = TRUE;
-				skk_buffer_set_j_status (buf, SKK_HENKAN);
-#endif
-				break;
-			default:
-				break;
-		}
-	} else {
-		switch (jstatus) {
-			case SKK_NONE:
-				/* nothing to do */
-				break;
-			case SKK_HENKAN:
-				/* nothing to do */
-				break;
-			case SKK_OKURI:
-				/* nothing to do */
-				break;
-			case SKK_CHOICE:
-				/* normal mode */
-#if 0
-				buf->store_result = TRUE;
-				skk_buffer_set_j_status (buf, SKK_NONE);
-#endif
-				break;
 			default:
 				break;
 		}
@@ -1194,24 +1264,24 @@ add_mode_check (SkkBuffer *buf, int ch)
 static gboolean
 add_char_jmode (SkkBuffer *buf, int ch)
 {
-	SkkJStatus jstatus;
-	DEBUG_DO (printf ("skkbuffer.c : Entering add_char_jmode\n"));
+	SkkQueryMode qmode;
 	if (!buf)
 		return FALSE;
 	add_mode_check (buf, ch);
 	/* again */
-	jstatus = skk_buffer_get_j_status (buf);
-	switch (jstatus) {
-		case SKK_NONE:
+	qmode = skk_buffer_get_query_mode (buf);
+	SKK_BUFFER_UNSET_FLAGS (buf, SKK_BUFFER_ALREADY_CLEARED);
+	switch (qmode) {
+		case SKK_QUERY_NONE:
 			return add_buffer_normal (buf, ch);
 			break;
-		case SKK_HENKAN:
+		case SKK_QUERY_BEGIN:
 			return add_buffer_henkan (buf, ch);
 			break;
-		case SKK_OKURI:
+		case SKK_QUERY_OKURI:
 			return add_buffer_okuri (buf, ch);
 			break;
-		case SKK_CHOICE:
+		case SKK_QUERY_DONE:
 			return add_buffer_choice (buf, ch);
 			break;
 		default:
@@ -1248,6 +1318,7 @@ add_char_abbrev (SkkBuffer *buf, int ch)
 	DEBUG_DO (printf ("skkbuffer.c : Entering add_char_abbrev\n"));
 	preedit = get_buf (buf, ch, FALSE);
 	if (preedit) {
+		SKK_BUFFER_UNSET_FLAGS (buf, SKK_BUFFER_ALREADY_CLEARED);
 		add_preedit_buf (buf, preedit);
 		if (isdigit (ch)) {
 			add_direction_word_int (buf, preedit);
@@ -1266,11 +1337,11 @@ add_char_abbrev (SkkBuffer *buf, int ch)
 static gboolean
 add_char (SkkBuffer *buf, int ch)
 {
-	SkkStatus status;
+	SkkMajorMode status;
 	DEBUG_DO (printf ("skkbuffer.c : Entering add_char\n"));
 	if (!buf)
 		return FALSE;
-	status = skk_buffer_get_status (buf);
+	status = skk_buffer_get_mode (buf);
 	/* FIXME */
 	switch (status) {
 		case SKK_J_MODE:
@@ -1325,12 +1396,12 @@ delete_backward_buf (SkkBuffer *buf)
 		buf->buf_len--;
 		if (skkbuf_blen(buf) == 0) {
 			set_buf (buf, NULL);
-			if (skk_buffer_get_j_status (buf) == SKK_HENKAN &&
+			if (skk_buffer_get_query_mode (buf) == SKK_QUERY_BEGIN &&
 					!skkbuf_p (buf)) {
 					skk_buffer_clear (buf);
-			} else if (skk_buffer_get_j_status (buf) == SKK_OKURI) {
+			} else if (skk_buffer_get_query_mode (buf) == SKK_QUERY_OKURI) {
 				if (!skkbuf_okuri (buf))
-					skk_buffer_set_j_status (buf, SKK_HENKAN);
+					skk_buffer_set_query_mode (buf, SKK_QUERY_BEGIN);
 			}	
 		}	
 		return TRUE;
@@ -1392,7 +1463,7 @@ delete_backward_preedit (SkkBuffer *buf)
 	buf->preedit_buf[skkbuf_plen (buf) - len] = '\0';
 	skkbuf_plen (buf) -= len;
 	if (skkbuf_plen (buf) == 0) {
-		skk_buffer_set_query_status (buf, QUERY_NONE);
+		skk_buffer_set_query_mode (buf, SKK_QUERY_NONE);
 		set_preedit_buf (buf, NULL);
 	} else {
 		preedit_emit (buf);
@@ -1403,12 +1474,12 @@ delete_backward_preedit (SkkBuffer *buf)
 static void
 delete_backward (SkkBuffer *buf)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering delete_backward\n"));
+	SkkQueryMode qmode;
 	if (!buf)
 		return;
-	/* if in SKK_NONE or SKK_HENKAN , then we must remove result */
-	if (skk_buffer_get_j_status (buf) == SKK_NONE
-	    || skk_buffer_get_j_status (buf) == SKK_HENKAN) {
+	qmode = skk_buffer_get_query_mode (buf);
+	/* if in SKK_QUERY_NONE or SKK_QUERY_BEGIN , then we must remove result */
+	if (qmode & (SKK_QUERY_NONE | SKK_QUERY_BEGIN)) {
 		set_result_buf (buf, NULL);
 	}
 	if (delete_backward_buf (buf))
@@ -1421,7 +1492,7 @@ delete_backward (SkkBuffer *buf)
 	if (!delete_backward_direction_word (buf)) {
 		DEBUG_DO (printf ("skkbuffer.c : delte_backward (delete direction_word failed\n"));
 	}
-	if (skk_buffer_get_query_status (buf) == QUERY_DONE) {
+	if (skk_buffer_get_query_mode (buf) == SKK_QUERY_DONE) {
 		skk_buffer_commit (buf, FALSE);
 		skk_buffer_clear (buf);
 	}
@@ -1546,7 +1617,7 @@ add_direction_word (SkkBuffer *buf, const gchar *append)
 		skkbuf_d (buf) = g_strdup (append);
 	}
 	skkbuf_dlen (buf) = strlen (skkbuf_d (buf));
-	buf->in_number = FALSE;
+	SKK_BUFFER_UNSET_FLAGS(buf, SKK_BUFFER_HAS_NUMBER);
 	DEBUG_DO (printf ("add_henkan %s\n",buf->direction_word));
 	return;
 }
@@ -1562,7 +1633,7 @@ add_direction_word_int (SkkBuffer *buf, const gchar *append)
 	DEBUG_DO (printf ("skkbuffer.c : Entering add_direction_word_int\n"));
 	if (!append)
 		return;
-	if (!buf->in_number) {
+	if (!skkbuf_has_num (buf)) {
 		if (skkbuf_d (buf)) {
 			if (cursor->caret) {
 				gchar *before = NULL;
@@ -1589,7 +1660,7 @@ add_direction_word_int (SkkBuffer *buf, const gchar *append)
 		}
 		buf->number_list = g_list_append (buf->number_list,g_strdup (append));
 		buf->has_num++;
-		buf->in_number = TRUE;
+		SKK_BUFFER_SET_FLAGS(buf, SKK_BUFFER_HAS_NUMBER);
 	} else {
 		GList *last;
 		gchar *last_buf;
@@ -1603,28 +1674,20 @@ add_direction_word_int (SkkBuffer *buf, const gchar *append)
 }
 
 static void
-set_status (SkkBuffer *buf,SkkStatus status)
+set_mode (SkkBuffer *buf,SkkMajorMode status)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering set_status\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering set_mode\n"));
 	if (!buf) return;
 	skk_buffer_clear (buf);
-	skk_mode_set_status (buf->mode,status);
+	skk_mode_set_mode (buf->mode,status);
 }
 
 static void
-set_j_status (SkkBuffer *buf,SkkJStatus jstatus)
+set_query_mode (SkkBuffer *buf,SkkQueryMode status)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering set_j_status\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering set_query_mode\n"));
 	if (!buf) return;
-	skk_mode_set_j_status (buf->mode,jstatus);
-}
-
-static void
-set_query_status (SkkBuffer *buf,SkkQueryStatus status)
-{
-	DEBUG_DO (printf ("skkbuffer.c : Entering set_query_status\n"));
-	if (!buf) return;
-	skk_mode_set_query_status (buf->mode,status);
+	skk_mode_set_query_mode (buf->mode,status);
 }
 
 static void
@@ -1632,6 +1695,8 @@ clear (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering clear\n"));
 	if (!buf) return;
+	if (skkbuf_already_cleared (buf))
+		return;
 	if (buf->mode) {
 		skk_mode_clear (buf->mode);
 	}
@@ -1650,7 +1715,7 @@ clear (SkkBuffer *buf)
 	if (buf->number_list) {
 		skk_utils_list_free (buf->number_list, TRUE, NULL, NULL);
 		buf->number_list = NULL;
-		buf->in_number = FALSE;
+		SKK_BUFFER_UNSET_FLAGS(buf, SKK_BUFFER_HAS_NUMBER);
 		buf->has_num = 0;
 	}
 	if (buf->completion_list) {
@@ -1670,36 +1735,42 @@ clear (SkkBuffer *buf)
 	if (skkbuf_numb (buf)) {
 		set_numb (buf, NULL);
 	}
+	if (skkbuf_dcomp (buf)) {
+		set_dcomp (buf, NULL);
+	}
 	if (skkbuf_cursor (buf)) {
 		skk_cursor_clear (skkbuf_cursor (buf));
 	}
+	buf->flag = 0;
 	buf->candidate_count = 0;
 	buf->candidate_max = 0;
-	buf->in_completion = FALSE;
 	skk_query_set_first (buf->query);
 	preedit_emit (buf);
+	SKK_BUFFER_SET_FLAGS (buf, SKK_BUFFER_ALREADY_CLEARED);
 }
 
 static void
 translate_henkan (SkkBuffer *buf)
 {
 	gchar *tmp_buf;
-	SkkQueryStatus qstatus;
+	SkkQueryMode qstatus;
 	DEBUG_DO (printf ("skkbuffer.c : Entering translate_henkan\n"));
-	qstatus = skk_buffer_get_query_status (buf);
+	qstatus = skk_buffer_get_query_mode (buf);
 	skk_mode_prepare_mark (buf->mode);
-	if (qstatus == QUERY_NORMAL) { 
+	if (qstatus == SKK_QUERY_BEGIN) { 
 		/* ֢ʡ */
 		tmp_buf = get_translate_buf (buf);
 		if (!tmp_buf) return; /* cannot find string */
 		add_preedit_buf (buf,tmp_buf);
 		add_direction_word (buf,tmp_buf);
 		g_free (tmp_buf);
-	} else if (qstatus == QUERY_DONE) {
+	} else if (qstatus == SKK_QUERY_DONE) {
 		/* ֢ʡ */
 		buf->preedit_buf = create_candidate (buf);
+#if 0
 		/*  ⡼ɤ˰ܹ */
 		skk_buffer_set_j_status (buf,SKK_CHOICE);
+#endif
 	}
 }
 
@@ -1708,11 +1779,11 @@ translate_okuri (SkkBuffer *buf)
 {
 	gchar *preedit;
 	DEBUG_DO (printf ("skkbuffer.c : Entering translate_okuri\n"));
-	skk_buffer_set_query_status (buf,QUERY_OKURI);
+	skk_buffer_set_query_mode (buf,SKK_QUERY_OKURI);
 	buf->henkan_okurigana = get_translate_buf (buf);
 	if (!buf->henkan_okurigana) return;
 	/* FIXME */
-	skk_buffer_set_query_status (buf,QUERY_DONE);
+	skk_buffer_set_query_mode (buf,SKK_QUERY_DONE);
 	preedit = create_candidate (buf);
 	/* FIXME put to enable undo */
 	if (buf->preedit_buf)
@@ -1720,7 +1791,6 @@ translate_okuri (SkkBuffer *buf)
 	buf->preedit_buf = g_strdup_printf ("%s%s",
 			preedit,buf->henkan_okurigana);
 	g_free (preedit);
-	skk_buffer_set_j_status (buf, SKK_CHOICE);
 }
 
 static gchar*
@@ -1735,23 +1805,23 @@ get_preedit_string (SkkBuffer *buf)
 		return NULL;
 	if (cursor->caret && skkbuf_p (buf)) {
 		if (separate_string_by_cursor (buf, skkbuf_p (buf), &before, &after)) {
-			if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-echo")) {
+			if (skk_conf_get_bool (skkbuf_conf (buf), "skk-echo")) {
 				ret = g_strdup_printf ("%s%s%s%s%s%s",
-				skk_mode_get_query_status_prefix (buf->mode) ?
-				skk_mode_get_query_status_prefix (buf->mode) : "",
+				skk_mode_get_query_mode_prefix (buf->mode) ?
+				skk_mode_get_query_mode_prefix (buf->mode) : "",
 				before ? before : "",
-				skk_mode_get_query_status_append (buf->mode) ?
-				skk_mode_get_query_status_append (buf->mode) : "",
+				skk_mode_get_query_mode_append (buf->mode) ?
+				skk_mode_get_query_mode_append (buf->mode) : "",
 				skkbuf_okuri (buf) ? skkbuf_okuri (buf) :  "",
 				skkbuf_b (buf) ? skkbuf_b (buf) : "",
 				after ? after : "");
 			} else {
 				ret = g_strdup_printf ("%s%s%s%s%s",
-				skk_mode_get_query_status_prefix (buf->mode) ?
-				skk_mode_get_query_status_prefix (buf->mode) : "",
+				skk_mode_get_query_mode_prefix (buf->mode) ?
+				skk_mode_get_query_mode_prefix (buf->mode) : "",
 				before ? before : "",
-				skk_mode_get_query_status_append (buf->mode) ?
-				skk_mode_get_query_status_append (buf->mode) : "",
+				skk_mode_get_query_mode_append (buf->mode) ?
+				skk_mode_get_query_mode_append (buf->mode) : "",
 				skkbuf_okuri (buf) ? skkbuf_okuri (buf) :  "",
 				after ? after : "");
 			}
@@ -1762,22 +1832,22 @@ get_preedit_string (SkkBuffer *buf)
 		}
 		return ret;
 	}
-	if (skk_conf_get_bool_by_name (skkbuf_conf (buf), "skk-echo")) {
+	if (skk_conf_get_bool (skkbuf_conf (buf), "skk-echo")) {
 		return g_strdup_printf ("%s%s%s%s%s",
-				skk_mode_get_query_status_prefix (buf->mode) ?
-				skk_mode_get_query_status_prefix (buf->mode) : "",
+				skk_mode_get_query_mode_prefix (buf->mode) ?
+				skk_mode_get_query_mode_prefix (buf->mode) : "",
 				skkbuf_p (buf) ? skkbuf_p (buf) : "",
-				skk_mode_get_query_status_append (buf->mode) ?
-				skk_mode_get_query_status_append (buf->mode) : "",
+				skk_mode_get_query_mode_append (buf->mode) ?
+				skk_mode_get_query_mode_append (buf->mode) : "",
 				skkbuf_okuri (buf) ? skkbuf_okuri (buf) :  "",
 				skkbuf_b (buf) ? skkbuf_b (buf) : "");
 	} else {
 		return g_strdup_printf ("%s%s%s%s",
-				skk_mode_get_query_status_prefix (buf->mode) ?
-				skk_mode_get_query_status_prefix (buf->mode) : "",
+				skk_mode_get_query_mode_prefix (buf->mode) ?
+				skk_mode_get_query_mode_prefix (buf->mode) : "",
 				skkbuf_p (buf) ? skkbuf_p (buf) : "",
-				skk_mode_get_query_status_append (buf->mode) ?
-				skk_mode_get_query_status_append (buf->mode) : "",
+				skk_mode_get_query_mode_append (buf->mode) ?
+				skk_mode_get_query_mode_append (buf->mode) : "",
 				skkbuf_okuri (buf) ? skkbuf_okuri (buf) : "");
 	}
 }
@@ -1812,7 +1882,7 @@ commit (SkkBuffer *buf, gboolean regist_dict)
 		add_result_buf (buf, after);
 	}
 	cursor->caret = store;
-	skk_buffer_set_j_status (buf, SKK_NONE);
+	skk_buffer_set_query_mode (buf, SKK_QUERY_NONE);
 	/* clear unused buffer */
 	set_okurigana (buf, NULL);
 	set_okuri_prefix (buf, NULL);
@@ -1827,28 +1897,19 @@ commit (SkkBuffer *buf, gboolean regist_dict)
 	result_emit (buf);
 }
 
-static SkkJStatus
-get_j_status (SkkBuffer *buf)
+static SkkMajorMode
+get_mode (SkkBuffer *buf)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering get_j_status\n"));
-	if (!buf)
-		return SKK_NONE;
-	return skk_mode_get_j_status (buf->mode);
-}
-
-static SkkStatus
-get_status (SkkBuffer *buf)
-{
-	DEBUG_DO (printf ("skkbuffer.c : Entering get_status\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering get_mode\n"));
 	if (!buf) return SKK_J_MODE;
-	return skk_mode_get_status (buf->mode);
+	return skk_mode_get_mode (buf->mode);
 }
 
 static gchar*
-get_status_string (SkkBuffer *buf)
+get_mode_string (SkkBuffer *buf)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering get_status_string\n"));
-	return skk_mode_get_status_string (buf->mode);
+	DEBUG_DO (printf ("skkbuffer.c : Entering get_mode_string\n"));
+	return skk_mode_get_mode_string (buf->mode);
 }
 
 static gchar*
@@ -1859,9 +1920,7 @@ get_translate_buf (SkkBuffer *buf)
 	gchar *jisx0201 = NULL;
 	gboolean katakana_state = FALSE;
 	DEBUG_DO (printf ("skkbuffer.c : Entering get_translate_buf\n"));
-	if (!buf) 
-		return NULL;
-	if (!skkbuf_b (buf))
+	if (!skk_buffer_has_buf (buf))
 		return NULL;
 	katakana_state = skk_mode_get_katakana (skkbuf_mode (buf));
 	ret = (gchar *) katakana_state ?
@@ -1976,16 +2035,16 @@ skk_buffer_translate (SkkBuffer *buf)
 		g_free (buf->result_buf);
 		buf->result_buf = NULL;
 	}
-	switch (skk_mode_get_j_status (buf->mode)) {
-		case SKK_NONE:
+	switch (skk_mode_get_query_mode (skkbuf_mode (buf))) {
+		case SKK_QUERY_NONE:
 			DEBUG_DO (printf ("direction_word SKK_NONE\n"));
 			skk_mode_prepare_mark (buf->mode);
 			buf->result_buf = get_translate_buf (buf);
 			break;
-		case SKK_HENKAN:
+		case SKK_QUERY_BEGIN:
 			translate_henkan (buf);
 			break;
-		case SKK_OKURI:
+		case SKK_QUERY_OKURI:
 			translate_okuri (buf);
 			/* NOT YET */
 			break;
@@ -2000,8 +2059,7 @@ skk_buffer_new (void)
 {
 	SkkBuffer* ret;
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_new\n"));
-	ret = g_new (SkkBuffer,1);
-	memset (ret,0,sizeof (SkkBuffer));
+	ret = g_new0 (SkkBuffer,1);
 	ret->mode = skk_mode_new ();
 	ret->conf = NULL;
 	ret->query = NULL;
@@ -2009,14 +2067,12 @@ skk_buffer_new (void)
 	ret->rule = NULL;
 	/* set default functions. */
 	ret->clear = clear;
-	ret->set_status = set_status;
-	ret->set_query_status = set_query_status;
-	ret->set_j_status = set_j_status;
-	ret->get_status = get_status;
-	ret->get_query_status = skk_buffer_get_query_status;
-	ret->get_j_status = get_j_status;
+	ret->set_mode = set_mode;
+	ret->set_query_mode = set_query_mode;
+	ret->get_mode = get_mode;
+	ret->get_query_mode = skk_buffer_get_query_mode;
 	ret->get_preedit_string = get_preedit_string;
-	ret->get_status_string = get_status_string;
+	ret->get_mode_string = get_mode_string;
 	ret->delete_backward = delete_backward;
 	ret->has_preedit = has_preedit;
 	ret->has_buf = has_buf;
@@ -2035,6 +2091,7 @@ void
 skk_buffer_destroy (SkkBuffer *buf)
 {
 	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_destroy\n"));
+	SKK_BUFFER_SET_FLAGS (buf, SKK_BUFFER_IN_DESTROY);
 	skk_buffer_clear (buf);
 	skk_mode_destroy (buf->mode);
 	skk_conf_destroy (buf->conf);
@@ -2068,10 +2125,10 @@ skk_buffer_destroy (SkkBuffer *buf)
 gboolean
 skk_buffer_add_char (SkkBuffer *buf, int ch)
 {
-	SkkStatus status;
+	SkkMajorMode status;
 	if (!buf) 
 		return FALSE;
-	status = skk_buffer_get_status (buf);	
+	status = skk_buffer_get_mode (buf);	
 
 	/* FIXME */
 	switch (status) {
@@ -2105,7 +2162,7 @@ skk_buffer_set_katakana (SkkBuffer *buf, gboolean state)
 	if (state != cur_state) {
 		/* reset status */
 		skk_buffer_clear (buf);
-		skk_mode_set_status (skkbuf_mode (buf), SKK_J_MODE);
+		skk_mode_set_mode (skkbuf_mode (buf), SKK_J_MODE);
 		skk_mode_set_katakana (skkbuf_mode (buf), state);
 	}
 	return;
@@ -2121,81 +2178,57 @@ skk_buffer_set_jisx0201_katakana (SkkBuffer *buf, gboolean state)
 	if (state != cur_state) {
 		/* reset status */
 		skk_buffer_clear (buf);
-		skk_mode_set_status (skkbuf_mode (buf), SKK_J_MODE);
+		skk_mode_set_mode (skkbuf_mode (buf), SKK_J_MODE);
 		skk_mode_set_jisx0201_katakana (skkbuf_mode (buf), state);
 	}
 	return;
 }
 
 void
-skk_buffer_set_query_status (SkkBuffer *buf, SkkQueryStatus status)
+skk_buffer_set_query_mode (SkkBuffer *buf, SkkQueryMode status)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_query_status\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_query_mode\n"));
 	if (!buf) 
 		return;
-	if (buf->set_query_status) {
-		buf->set_query_status (buf,status);
+	if (buf->set_query_mode) {
+		buf->set_query_mode (buf,status);
 	}
 	return;
 }
 
-SkkQueryStatus
-skk_buffer_get_query_status (SkkBuffer *buf)
+SkkQueryMode
+skk_buffer_get_query_mode (SkkBuffer *buf)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_query_status\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_query_mode\n"));
 	if (!buf) 
-		return QUERY_NONE;
-	return skk_mode_get_query_status (buf->mode);
+		return SKK_QUERY_NONE;
+	return skk_mode_get_query_mode (buf->mode);
 }
 
 void
-skk_buffer_set_status (SkkBuffer *buf, SkkStatus status)
+skk_buffer_set_mode (SkkBuffer *buf, SkkMajorMode status)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_status\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_mode\n"));
 	if (!buf)
 		return;
-	if (buf->set_status) {
-		buf->set_status (buf,status);
+	if (buf->set_mode) {
+		buf->set_mode (buf,status);
 	}
 }
 
-SkkStatus
-skk_buffer_get_status (SkkBuffer *buf)
+SkkMajorMode
+skk_buffer_get_mode (SkkBuffer *buf)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_status\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_mode\n"));
 	if (!buf) 
 		return SKK_J_MODE;
-	if (buf->get_status) {
-		return buf->get_status (buf);
+	if (buf->get_mode) {
+		return buf->get_mode (buf);
 	} else {
 		return SKK_J_MODE;
 	}
 }
 
-void
-skk_buffer_set_j_status (SkkBuffer *buf, SkkJStatus jstatus)
-{
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_set_j_status\n"));
-	if (!buf) 
-		return;
-	if (buf->set_j_status) {
-		buf->set_j_status (buf,jstatus);
-	}
-}
-
-SkkJStatus
-skk_buffer_get_j_status (SkkBuffer *buf)
-{
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_j_status\n"));
-	if (!buf) 
-		return SKK_NONE;
-	if (buf->get_j_status) {
-		return buf->get_j_status (buf);
-	} else {
-		return SKK_NONE;
-	}
-}
-
 void
 skk_buffer_clear (SkkBuffer *buf)
 {
@@ -2471,14 +2504,14 @@ skk_buffer_get_preedit_string (SkkBuffer *buf)
 }
 
 const gchar*
-skk_buffer_get_status_string (SkkBuffer *buf)
+skk_buffer_get_mode_string (SkkBuffer *buf)
 {
-	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_status_string\n"));
+	DEBUG_DO (printf ("skkbuffer.c : Entering skk_buffer_get_mode_string\n"));
 	/* FIXME OK ? */
 	if (!buf) 
 		return g_strdup("Unknown");
-	if (buf->get_status_string) {
-		return buf->get_status_string (buf);
+	if (buf->get_mode_string) {
+		return buf->get_mode_string (buf);
 	} else {
 		return g_strdup ("Unknown");
 	}
@@ -2707,27 +2740,27 @@ skk_buffer_set_conf (SkkBuffer *buf, SkkConf *conf)
 	}
 	buf->conf = conf;
 	skk_query_set_conf (buf->query, buf->conf);
-	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-abbrev-mode-string"))) {
+	if ((status_str = skk_conf_get_string (skkbuf_conf (buf), "skk-abbrev-mode-string"))) {
 		skk_mode_set_abbrev_str (skkbuf_mode (buf), status_str);
 		g_free (status_str);
 	}
-	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-hirakana-mode-string"))) {
+	if ((status_str = skk_conf_get_string (skkbuf_conf (buf), "skk-hirakana-mode-string"))) {
 		skk_mode_set_hirakana_str (skkbuf_mode (buf), status_str);
 		g_free (status_str);
 	}
-	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-katakana-mode-string"))) {
+	if ((status_str = skk_conf_get_string (skkbuf_conf (buf), "skk-katakana-mode-string"))) {
 		skk_mode_set_katakana_str (skkbuf_mode (buf), status_str);
 		g_free (status_str);
 	}
-	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-jisx0201-kana-mode-string"))) {
+	if ((status_str = skk_conf_get_string (skkbuf_conf (buf), "skk-jisx0201-kana-mode-string"))) {
 		skk_mode_set_jisx0201_kana_str (skkbuf_mode (buf), status_str);
 		g_free (status_str);
 	}
-	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-jisx0208-latin-mode-string"))) {
+	if ((status_str = skk_conf_get_string (skkbuf_conf (buf), "skk-jisx0208-latin-mode-string"))) {
 		skk_mode_set_jisx0208_latin_str (skkbuf_mode (buf), status_str);
 		g_free (status_str);
 	}
-	if ((status_str = skk_conf_get_string_by_name (skkbuf_conf (buf), "skk-latin-mode-string"))) {
+	if ((status_str = skk_conf_get_string (skkbuf_conf (buf), "skk-latin-mode-string"))) {
 		skk_mode_set_latin_str (skkbuf_mode (buf), status_str);
 		g_free (status_str);
 	}
@@ -2887,79 +2920,77 @@ skk_buffer_add_dict_with_value (SkkBuffer *buf, gchar *value)
 	return;
 }
 
-void
+gboolean
 skk_buffer_get_completion (SkkBuffer *buf)
 {
 	gchar *value;
-	GList *append = NULL;
-	if (!buf)
-		return;
-	if (buf->completion_list) {
-		skk_utils_list_free (buf->completion_list, TRUE, NULL, NULL);
-		buf->completion_count = 0;
-		buf->completion_list = NULL;
-	}
-	if (!skkbuf_d (buf))
-		return;
-	/* initialized before completion */
-	skk_query_set_first (buf->query);
-	buf->completion_list = skk_query_do_completion (buf->query, skkbuf_d (buf));
-	while (skk_query_set_next (buf->query)) {
-		append = skk_query_do_completion (buf->query, skkbuf_d (buf));
-		if (append)
-			buf->completion_list = skk_utils_list_merge_string (buf->completion_list, append, TRUE);
-	}
-	/* re-initialized */
-	skk_query_set_first (buf->query);
-	if (!buf->completion_list)
-		return;
+	if (!get_completion (buf))
+		return FALSE;
 	value = (gchar *)g_list_nth_data (buf->completion_list, 0);
 	if (!value)
-		return;
+		return FALSE;
 	set_direction_word (buf, value);
 	set_preedit_buf (buf, value);
-	buf->in_completion = TRUE;
+	SKK_BUFFER_SET_FLAGS(buf, SKK_BUFFER_IN_COMPLETION);
 	preedit_emit (buf);
 	/* TODO */
 	/* Translate num? */
+	return TRUE;
 }
 
-void
+gboolean
 skk_buffer_set_next_completion (SkkBuffer *buf)
 {
 	gchar *value;
 	if (!buf)
-		return;
-	if (!buf->completion_list)
-		return;
+		return FALSE;
+	if (!skkbuf_comp_list (buf))
+		return FALSE;
 	buf->completion_count++;
 	value = g_list_nth_data (buf->completion_list, buf->completion_count);
 	if (!value) {
-		buf->completion_count--;
-		return;
+		if (skk_conf_get_bool (skkbuf_conf (buf), "skk-comp-circulate")) {
+			buf->completion_count = 0;
+			value = g_list_nth_data (buf->completion_list, buf->completion_count);
+			if (!value)
+				return FALSE;
+		} else {
+			buf->completion_count--;
+			return FALSE;
+		}
 	}
 	set_direction_word (buf, value);
 	set_preedit_buf (buf, value);
 	preedit_emit (buf);
+	return TRUE;
 }
 
-void
+gboolean
 skk_buffer_set_prev_completion (SkkBuffer *buf)
 {
 	gchar *value;
 	if (!buf)
-		return;
+		return FALSE;
 	if (!skkbuf_comp_list (buf))
-		return;
-	if (skkbuf_comp_count (buf) < 1)
-		return;
-	buf->completion_count--;
+		return FALSE;
+	if (skk_conf_get_bool (skkbuf_conf (buf), "skk-comp-circulate")) {
+		if (!skkbuf_comp_count (buf)) {
+			skkbuf_comp_count (buf) = g_list_length (skkbuf_comp_list (buf)) - 1;
+		} else {
+			buf->completion_count--;
+		}
+	} else {
+		if (skkbuf_comp_count (buf) < 1)
+			return FALSE;
+		buf->completion_count--;
+	}
 	value = (gchar *)g_list_nth_data (buf->completion_list, buf->completion_count);
 	if (!value) /* may not reached */
-		return;
+		return FALSE;
 	set_direction_word (buf, value);
 	set_preedit_buf (buf, value);
 	preedit_emit (buf);
+	return TRUE;
 }
 
 void
