@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: skkconf.c,v 1.9 2003/02/19 17:44:51 famao Exp $ */
+/* $Id: skkconf.c,v 1.3.2.61 2003/04/17 06:19:22 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -50,129 +50,31 @@
 #include "skkconf.h"
 #include "skkconf_private.h"
 
+static gboolean table_clear_func (gpointer key, gpointer value, gpointer user_data);
 static void clear (SkkConf *conf);
 
 /* Parse Item */
 static void parse_item_one (SkkConf *conf, xmlNodePtr node);
 static void parse_item (SkkConf *conf, xmlDocPtr doc);
-/* Parse Rule */
-static void parse_rule_one (SkkConf *conf, xmlNodePtr node);
-static void parse_rule (SkkConf *conf, xmlDocPtr doc);
-static void parse_rule_delete (SkkConf *conf, xmlDocPtr doc);
 
 static SkkConfType find_type (gchar *str);
-static SkkConfItemType find_item_type (const gchar *str);
-static SkkConfItem *find_item (GList *list, SkkConfItemType type);
 
 /* Add Item to List */
-static void add_item_bool (SkkConf *conf, SkkConfItem *item);
-static void add_item_string (SkkConf *conf, SkkConfItem *item);
-static void add_item_num (SkkConf *conf, SkkConfItem *item);
 static void add_defaults (SkkConf* conf);
 
 /* Find Item */
-static gboolean find_item_func (gconstpointer p1, gconstpointer p2);
-static gint find_item_each_for_delete (gconstpointer p1, gconstpointer p2);
-static GList *find_item_for_delete (GList *list, SkkConfItem *item);
 
 /* Load Configuration */
 static void load_rc (SkkConf *conf, const gchar *filename);
 
-/* Event */
-static void add_rule_emit (SkkConf *conf, const gchar *key, const gchar *hira,
-		const gchar *kata, const gchar *append);
-static void delete_rule_emit (SkkConf *conf, const gchar *key);
+typedef struct _SkkConfCallbackPrivate SkkConfCallbackPrivate;
 
-typedef struct _SkkConfAddRuleFunc SkkConfAddRuleFunc;
-typedef struct _SkkConfDeleteRuleFunc SkkConfDeleteRuleFunc;
-
-struct _SkkConfAddRuleFunc {
-	SkkConfAddRuleListener listener;
+struct _SkkConfCallbackPrivate {
+	gchar *key;
+	SkkConfCallback callback;
 	gpointer user_data;
 };
 
-struct _SkkConfDeleteRuleFunc {
-	SkkConfDeleteRuleListener listener;
-	gpointer user_data;
-};
-
-static void
-add_rule_emit (SkkConf *conf, const gchar *key, const gchar *hira,
-		const gchar *kata, const gchar *append)
-{
-	GSList *tmp_list;
-	SkkConfAddRuleFunc *func;
-	if (!conf)
-		return;
-	for (tmp_list = conf->conf_add_rule_listener;
-			tmp_list; tmp_list = g_slist_next (tmp_list)) {
-		func = (SkkConfAddRuleFunc *)tmp_list->data;
-		if (func) {
-			func->listener (conf, key, hira, kata, append, func->user_data);
-		}
-	}
-	return;
-}
-
-static void
-delete_rule_emit (SkkConf *conf, const gchar *key)
-{
-	GSList *tmp_list;
-	SkkConfDeleteRuleFunc *func;
-	if (!conf)
-		return;
-	for (tmp_list = conf->conf_delete_rule_listener;
-			tmp_list; tmp_list = g_slist_next (tmp_list)) {
-		func = (SkkConfDeleteRuleFunc *)tmp_list->data;
-		if (func) {
-			func->listener (conf, key, func->user_data);
-		}
-
-	}
-	return;
-}
-
-static gboolean
-find_item_func (gconstpointer p1, gconstpointer p2)
-{
-	SkkConfItem *item;
-	if (!p1)
-		return TRUE;
-	if (!p2)
-		return TRUE;
-	item = (SkkConfItem *)p1;
-	if (GPOINTER_TO_INT (p2) == item->itemtype) {
-		return FALSE;
-	}
-	return TRUE;
-	
-}
-
-static gint
-find_item_each_for_delete (gconstpointer a, gconstpointer b)
-{
-	if (!a)
-		return 1;
-	if (!b)
-		return 1;
-	if (((SkkConfItem *)a)->itemtype == ((SkkConfItem *)b)->itemtype)
-		return 0;
-	return 1;
-}
-
-static GList *
-find_item_for_delete (GList *list, SkkConfItem *item)
-{
-	GList *ret = NULL;
-	if (!list)
-		return NULL;
-	ret = g_list_find_custom (list, item, find_item_each_for_delete);
-	if (ret) {
-		return ret;
-	}
-	return NULL;
-}
-
 static void
 add_defaults (SkkConf* conf)
 {
@@ -182,53 +84,6 @@ add_defaults (SkkConf* conf)
 	}
 }
 
-static void
-add_item_bool (SkkConf *conf, SkkConfItem *item)
-{
-	GList *found;
-	if (item->user_defined) {
-		found = find_item_for_delete (conf->conf_bool, item);
-		if (found) {
-			skk_conf_item_destroy ((SkkConfItem*)found->data);
-			conf->conf_bool = g_list_remove_link (conf->conf_bool, found);
-			g_list_free_1 (found);
-		}
-	}
-	conf->conf_bool = g_list_prepend (conf->conf_bool, item);
-	return;
-}
-
-static void
-add_item_string (SkkConf *conf, SkkConfItem *item)
-{
-	GList *found;
-	if (item->user_defined) {
-		found = find_item_for_delete (conf->conf_string, item);
-		if (found) {
-			skk_conf_item_destroy ((SkkConfItem *)found->data);
-			conf->conf_string = g_list_remove_link (conf->conf_string, found);
-			g_list_free_1 (found);
-		}
-	}
-	conf->conf_string = g_list_prepend (conf->conf_string, item);
-	return;
-}
-
-static void
-add_item_num (SkkConf *conf, SkkConfItem *item)
-{
-	GList *found;
-	if (item->user_defined) {
-		found = find_item_for_delete (conf->conf_num, item);
-		if (found) {
-			skk_conf_item_destroy ((SkkConfItem *)found->data);
-			conf->conf_num = g_list_remove_link (conf->conf_num, found);
-			g_list_free_1 (found);
-		}
-	}
-	conf->conf_num = g_list_prepend (conf->conf_num, item);
-	return;
-}
 
 static SkkConfType
 find_type (gchar *str)
@@ -246,22 +101,50 @@ find_type (gchar *str)
 	return SKKCONF_TYPE_INVALID;
 }
 
-static gint
-find_item_type_p (gconstpointer p1, gconstpointer p2)
+static void
+confvalue_free (gpointer data, gpointer user_data)
 {
-	return g_strcasecmp ((const gchar *)p1, ((SkkConfItem*)p2)->name);
+	SkkConfValue *v = (SkkConfValue *)data;
+	g_free (v->name);
+	if (v->value)
+		g_free (v->value);
 }
 
-static SkkConfItemType
-find_item_type (const gchar *str)
+static void
+parse_element_one (SkkConf *conf, xmlNodePtr node, SkkConfCallbackPrivate *p)
 {
-	SkkConfItem *item;
-	item = bsearch (str, itemp, SKKCONF_DEFAULT_ITEM_SIZE,
-			sizeof (SkkConfItem), find_item_type_p);
-	if (item)
-		return item->itemtype;
+	GSList *l = NULL;
+	SkkConfValue *v = NULL;
+	xmlNodePtr child;
+	xmlChar *type = NULL;
+	xmlChar *value = NULL;
+	child = node->xmlChildrenNode;
+	if (!child)
+		return;
+	type = xmlGetProp (node, "type");
+	for (; child; child = child->next) {
+		if (xmlIsBlankNode (node)) {
+			continue;
+		}
+		if (!child->name)
+			continue;
+		v = g_new0 (SkkConfValue, 1);
+		v->name = g_strdup (child->name);
+		value = xmlNodeGetContent (child);
+		if (value) {
+			v->value = skk_utils_utf8_to_eucjp (value);
+			g_free (value);
+		} else {
+			v->value = NULL;
+		}
+		l = g_slist_append (l, v);
+	}
+	p->callback (conf, l, type, p->user_data);
+	if (type)
+		g_free (type);
+	skk_utils_slist_free (l, TRUE, confvalue_free, NULL);
+	return;
 
-	return SKKCONF_ITEM_INVALID;
 }
 
 static void
@@ -276,7 +159,6 @@ parse_item_one (SkkConf *conf, xmlNodePtr node)
 	gpointer value_e = NULL;
 	SkkConfItem *item;
 	SkkConfType ctype;
-	SkkConfItemType itype;
 	if (!node)
 		return;
 	child = node->xmlChildrenNode;
@@ -301,10 +183,6 @@ parse_item_one (SkkConf *conf, xmlNodePtr node)
 			info = xmlNodeGetContent (child);
 		}
 	}
-	itype = find_item_type (name);
-	if (itype == SKKCONF_ITEM_INVALID) {
-		goto err;
-	}
 	info_e = info ? skk_utils_utf8_to_eucjp (info) : NULL;
 	switch (ctype) {
 		case SKKCONF_TYPE_BOOL:
@@ -329,7 +207,7 @@ parse_item_one (SkkConf *conf, xmlNodePtr node)
 			g_assert (0); /* not reached */
 			break;
 	}
-	item = skk_conf_item_new_with_value (ctype, itype, name, info_e, value_e);
+	item = skk_conf_item_new_with_value (ctype, name, info_e, value_e);
 	skk_conf_add_item (conf, item);
 	if (name)
 		xmlFree (name);
@@ -344,34 +222,22 @@ parse_item_one (SkkConf *conf, xmlNodePtr node)
 	if (value_e && ctype == SKKCONF_TYPE_STRING)
 		g_free (value_e);
 	return;
-
-err:
-	if (name)
-		xmlFree (name);
-	if (value)
-		xmlFree (value);
-	if (type)
-		xmlFree (type);
-	if (info)
-		xmlFree (info);
-	return;
 }
 
 static void
-parse_rule_delete (SkkConf *conf, xmlDocPtr doc)
+parse_element (SkkConf *conf, xmlDocPtr doc, SkkConfCallbackPrivate *p)
 {
 	xmlXPathContextPtr context;
 	xmlXPathObjectPtr object;
 	xmlNodeSetPtr nodeset = NULL;
 	xmlNodePtr node = NULL;
-	xmlChar *key = NULL;
 	gint i = 0;
 	if (!doc)
 		return;
 	context = xmlXPathNewContext (doc);
 	if (!context)
 		return;
-	object = xmlXPathEval ("/iiimf-skk/delete-rule/key", context);
+	object = xmlXPathEval (p->key, context);
 	if (!object) {
 		xmlXPathFreeContext (context);
 		return;
@@ -388,12 +254,7 @@ parse_rule_delete (SkkConf *conf, xmlDocPtr doc)
 		return;
 	}
 	for (node = nodeset->nodeTab[0]; i < nodeset->nodeNr ; node = nodeset->nodeTab[i]) {
-		key = xmlNodeGetContent (node);
-		if (key) {
-			if (*key) 
-				delete_rule_emit (conf, key);
-			xmlFree (key);
-		}
+		parse_element_one (conf, node, p);
 		i++;
 	}
 	xmlXPathFreeObject (object);
@@ -439,129 +300,18 @@ parse_item (SkkConf *conf, xmlDocPtr doc)
 	return;
 }
 
-static void
-parse_rule_one (SkkConf *conf, xmlNodePtr node)
+static gboolean
+table_clear_func (gpointer key, gpointer value, gpointer user_data)
 {
-	xmlNodePtr child;
-	xmlChar *key = NULL;
-	xmlChar *hiragana = NULL;
-	xmlChar *katakana = NULL;
-	xmlChar *append = NULL;
-	gchar *hira_e = NULL;
-	gchar *kata_e = NULL;
-	gchar *append_e = NULL;
-	if (!node)
-		return;
-	child = node->xmlChildrenNode;
-	if (!child)
-		return;
-	/*
-	 * NOTICE
-	 * we have DTD (<!ELEMENT rule (key, hiragana, katakana, append)>
-	 * when any modification is added to DTD, FIXME.
-	 * famao: currently, dont't believe DTD.
-	 */
-	for (; child ; child = child->next) {
-		if (xmlIsBlankNode (node)) {
-			continue;
-		}
-		if (!xmlStrcmp ("key", child->name)) {
-			key = xmlNodeGetContent (child);
-		} else if (!xmlStrcmp ("hiragana", child->name)) {
-			hiragana = xmlNodeGetContent (child);
-		} else if (!xmlStrcmp ("katakana", child->name)) {
-			katakana = xmlNodeGetContent (child);
-		} else if (!xmlStrcmp ("append", child->name)) {
-			append = xmlNodeGetContent (child);
-		}
-	}
-	/* FIXME add rule */
-	hira_e = skk_utils_utf8_to_eucjp (hiragana);
-	kata_e = skk_utils_utf8_to_eucjp (katakana);
-	append_e = skk_utils_utf8_to_eucjp (append);
-	add_rule_emit (conf, key, hira_e, kata_e, append_e);
-#if 0
-	rule = skk_conv_rule_item_new_with_value (key, hira_e, kata_e, append_e);
-	skk_conv_add_rule (rule);
-#endif
-	if (key)
-		xmlFree (key);
-	if (hiragana)
-		xmlFree (hiragana);
-	if (katakana)
-		xmlFree (katakana);
-	if (append)
-		xmlFree (append);
-	if (hira_e)
-		g_free (hira_e);
-	if (kata_e)
-		g_free (kata_e);
-	if (append_e)
-		g_free (append_e);
-	return;
-}
-
-static void
-parse_rule (SkkConf *conf, xmlDocPtr doc)
-{
-	xmlXPathContextPtr context;
-	xmlXPathObjectPtr object;
-	xmlNodeSetPtr nodeset = NULL;
-	xmlNodePtr node = NULL;
-	gint i = 0;
-	if (!doc)
-		return;
-	context = xmlXPathNewContext (doc);
-	if (!context)
-		return;
-	object = xmlXPathEval ("/iiimf-skk/rule", context);
-	if (!object) {
-		xmlXPathFreeContext (context);
-		return;
-	}
-	if (object->type == XPATH_NODESET) {
-		nodeset = object->nodesetval;
-	}
-	/*
-	 * NOTICE:
-	 * nodesetval is not allocated by libxml2, so don't free after using it
-	 */
-	if (!nodeset || !nodeset->nodeTab) {
-		xmlXPathFreeObject (object);
-		return;
-	}
-	for (node = nodeset->nodeTab[0]; i < nodeset->nodeNr ; node = nodeset->nodeTab[i]) {
-		parse_rule_one (conf, node);
-		i++;
-	}
-	xmlXPathFreeObject (object);
-	xmlXPathFreeContext (context);
-	return;
-}
-
-static void
-list_free_func (gpointer data, gpointer user_data)
-{
-	if (data) {
-		skk_conf_item_destroy ((SkkConfItem *)data);
-	}
+	if (value)
+		skk_conf_item_destroy ((SkkConfItem *)value);
+	return TRUE;
 }
 
 static void
 clear (SkkConf *conf)
 {
-	if (conf->conf_num) {
-		skk_utils_list_free (conf->conf_num, TRUE, list_free_func, NULL);
-		conf->conf_num = NULL;
-	}
-	if (conf->conf_string) {
-		skk_utils_list_free (conf->conf_string, TRUE, list_free_func, NULL);
-		conf->conf_string = NULL;
-	}
-	if (conf->conf_bool) {
-		skk_utils_list_free (conf->conf_bool, TRUE, list_free_func, NULL);
-		conf->conf_bool = NULL;
-	}
+	g_hash_table_foreach_remove (conf->conf_table, table_clear_func, NULL);
 	return;
 }
 
@@ -570,6 +320,7 @@ load_rc (SkkConf *conf, const char *filename)
 {
 	xmlDocPtr doc = NULL;
 	xmlNodePtr root = NULL;
+	GSList *callbacks;
 	struct stat s;
 	if (stat (filename, &s) == -1)
 		return;
@@ -587,8 +338,9 @@ load_rc (SkkConf *conf, const char *filename)
 		xmlFreeDoc (doc);
 		return;
 	}
-	parse_rule_delete (conf, doc);
-	parse_rule (conf, doc);
+	for (callbacks = conf->callbacks; callbacks; callbacks = g_slist_next (callbacks)) {
+		parse_element (conf, doc, callbacks->data);
+	}
 	parse_item (conf, doc);
 	xmlFreeDoc(doc);
 }
@@ -602,9 +354,13 @@ skk_conf_load_rc (SkkConf *conf, char *user_name)
 	strcpy(fname, "sample.config.xml");
 #else
 	struct passwd *pw;
-	if ((pw = getpwnam(user_name)) == NULL)
-		return;
-	snprintf (fname, FILENAME_MAX, "%s/.iiimf-skk/config.xml", pw->pw_dir);
+	if (!user_name)
+		strcpy (fname, "sample.config.xml");
+	else {
+		if ((pw = getpwnam(user_name)) == NULL)
+			return;
+		snprintf (fname, FILENAME_MAX, "%s/.iiimf-skk/config.xml", pw->pw_dir);
+	}
 #endif
 	load_rc (conf, fname);
 }
@@ -617,17 +373,21 @@ skk_conf_new (void)
 	ret = g_new (SkkConf, 1);
 	memset (ret, 0, sizeof (SkkConf));
 	ret->clear = clear;
-	ret->conf_string = NULL;
-	ret->conf_bool = NULL;
-	ret->conf_num = NULL;
-	ret->conf_add_rule_listener = NULL;
-	ret->conf_delete_rule_listener = NULL;
+	ret->conf_table = g_hash_table_new (g_str_hash, g_str_equal);
+	ret->callbacks = NULL;
 	/* initialize */
 	add_defaults (ret);
 	skk_conf_ref (ret);
 	return ret;
 }
 
+static void
+callbacks_free (gpointer data, gpointer user_data)
+{
+	SkkConfCallbackPrivate *p = (SkkConfCallbackPrivate *)data;
+	g_free (p->key);
+}
+
 void
 skk_conf_destroy (SkkConf *conf)
 {
@@ -638,12 +398,10 @@ skk_conf_destroy (SkkConf *conf)
 		return;
 	}
 	skk_conf_clear (conf);
-	if (conf->conf_add_rule_listener) {
-		skk_utils_slist_free (conf->conf_add_rule_listener, TRUE, NULL, NULL);
-	}
-	if (conf->conf_delete_rule_listener) {
-		skk_utils_slist_free (conf->conf_delete_rule_listener, TRUE, NULL, NULL);
+	if (conf->callbacks) {
+		skk_utils_slist_free (conf->callbacks, TRUE, callbacks_free, NULL);
 	}
+	g_hash_table_destroy (conf->conf_table);
 	g_free (conf);
 }
 
@@ -681,61 +439,35 @@ skk_conf_unref (SkkConf *conf)
 void
 skk_conf_add_item (SkkConf *conf, SkkConfItem *item)
 {
-	if (!conf || !item)
+	SkkConfItem *removed;
+	gchar *key;
+	if (!conf || !item || !item->name)
 		return;
-	switch (item->type) {
-		case SKKCONF_TYPE_BOOL:
-			add_item_bool (conf, item);
-			break;
-		case SKKCONF_TYPE_STRING:
-			add_item_string (conf, item);
-			break;
-		case SKKCONF_TYPE_NUM:
-			add_item_num (conf, item);
-			break;
-		default:
-			return;
-			break;
+	if (g_hash_table_lookup_extended (conf->conf_table,
+				(gconstpointer)item->name, 
+				(gpointer *)&key, (gpointer *)&removed)) {
+		g_hash_table_remove (conf->conf_table, item->name);
+		skk_conf_item_destroy (removed);
 	}
+	g_hash_table_insert (conf->conf_table, item->name, item);
 	return;
 }
 
-static SkkConfItem *
-find_item (GList *list, SkkConfItemType type)
-{
-	GList *ret;
-	if (!list)
-		return NULL;
-	ret = g_list_find_custom (list, GINT_TO_POINTER (type), find_item_func);
-	if (!ret)
-		return NULL;
-	return (SkkConfItem *)ret->data;
-}
-
 void
-skk_conf_set_bool (SkkConf *conf, SkkConfItemType type, gboolean value)
+skk_conf_set_bool (SkkConf *conf, const gchar *name, gboolean value)
 {
 	SkkConfItem *item;
-	/* TODO How to */
-	if (!conf)
+	if (!name)
 		return;
-	item = find_item (conf->conf_bool, type);
-	if (!item)
+	item = g_hash_table_lookup (conf->conf_table, name);
+	if (!item || (item->type != SKKCONF_TYPE_BOOL))
 		return;
 	item->value = GINT_TO_POINTER (value);
 	return;
 }
 
 void
-skk_conf_set_bool_by_name (SkkConf *conf, const gchar *name, gboolean value)
-{
-	if (!name)
-		return;
-	skk_conf_set_bool (conf, find_item_type (name), value);
-}
-
-void
-skk_conf_set_string (SkkConf *conf, SkkConfItemType type, const gchar *string)
+skk_conf_set_string (SkkConf *conf, const gchar *name, const gchar *string)
 {
 	SkkConfItem *item;
 	gchar *value = NULL;
@@ -744,8 +476,10 @@ skk_conf_set_string (SkkConf *conf, SkkConfItemType type, const gchar *string)
 		return;
 	if (!string)
 		return;
-	item = find_item (conf->conf_string, type);
-	if (!item)
+	if (!name)
+		return;
+	item = g_hash_table_lookup (conf->conf_table, name);
+	if (!item || item->type != SKKCONF_TYPE_STRING)
 		return;
 	if (item->value) {
 		value = item->value;
@@ -757,53 +491,32 @@ skk_conf_set_string (SkkConf *conf, SkkConfItemType type, const gchar *string)
 }
 
 void
-skk_conf_set_string_by_name (SkkConf *conf, const gchar *name, const gchar *string)
-{
-	if (!name)
-		return;
-	skk_conf_set_string (conf, find_item_type (name), string);
-}
-
-void
-skk_conf_set_num (SkkConf *conf, SkkConfItemType type, gint num)
+skk_conf_set_num (SkkConf *conf, const gchar *name, gint num)
 {
 	SkkConfItem *item;
 	/* TODO How to */
 	if (!conf)
 		return;
-	item = find_item (conf->conf_num, type);
-	if (!item)
+	if (!name)
+		return;
+	item = g_hash_table_lookup (conf->conf_table, name);
+	if (!item || item->type != SKKCONF_TYPE_NUM)
 		return;
 	item->value = GINT_TO_POINTER (num);
 	return;
 }
 
-void
-skk_conf_set_num_by_name (SkkConf *conf, const gchar *name, gint num)
-{
-	if (!name)
-		return;
-	skk_conf_set_num (conf, find_item_type (name), num);
-}
-
 gboolean
-skk_conf_get_bool_by_name (SkkConf *conf, const gchar *name)
-{
-	if (!conf)
-		return FALSE;
-	if (!name)
-		return FALSE;
-	return skk_conf_get_bool (conf, find_item_type (name));
-}
-
-gboolean
-skk_conf_get_bool (SkkConf *conf, SkkConfItemType type)
+skk_conf_get_bool (SkkConf *conf, const gchar *name)
 {
 	SkkConfItem *item;
 	if (!conf)
 		return FALSE;
-	item = find_item (conf->conf_bool, type);
-	if (!item)
+	if (!name)
+		return FALSE;
+
+	item = g_hash_table_lookup (conf->conf_table, name);
+	if (!item || item->type != SKKCONF_TYPE_BOOL)
 		return FALSE;
 	else
 		return GPOINTER_TO_INT (item->value);
@@ -811,47 +524,31 @@ skk_conf_get_bool (SkkConf *conf, SkkConfItemType type)
 }
 
 gchar *
-skk_conf_get_string_by_name (SkkConf *conf, const gchar *name)
-{
-	if (!conf)
-		return NULL;
-	if (!name)
-		return NULL;
-	return skk_conf_get_string (conf, find_item_type (name));
-}
-
-gchar *
-skk_conf_get_string (SkkConf *conf, SkkConfItemType type)
+skk_conf_get_string (SkkConf *conf, const gchar *name)
 {
 	SkkConfItem *item;
 	if (!conf)
 		return NULL;
-	item = find_item (conf->conf_string, type);
-	if (!item || !item->value)
+	if (!name)
 		return NULL;
-	else
+	item = g_hash_table_lookup (conf->conf_table, name);
+	if (!item || item->type != SKKCONF_TYPE_STRING || !item->value)
+		return NULL;
+	else 
 		return g_strdup ((gchar *)item->value);
 	return NULL;
 }
 
 gint
-skk_conf_get_num_by_name (SkkConf *conf, const gchar *name)
-{
-	if (!conf)
-		return -1;
-	if (!name)
-		return -1;
-	return skk_conf_get_num (conf, find_item_type (name));
-}
-
-gint
-skk_conf_get_num (SkkConf *conf, SkkConfItemType type)
+skk_conf_get_num (SkkConf *conf, const gchar *name)
 {
 	SkkConfItem *item;
 	if (!conf)
 		return -1;
-	item = find_item (conf->conf_num, type);
-	if (!item)
+	if (!name)
+		return -1;
+	item = g_hash_table_lookup (conf->conf_table, name);
+	if (!item || item->type != SKKCONF_TYPE_NUM)
 		return -1;
 	else
 		return GPOINTER_TO_INT (item->value);
@@ -874,12 +571,11 @@ skk_conf_item_new (void)
 }
 
 SkkConfItem*
-skk_conf_item_new_with_value (SkkConfType type, SkkConfItemType itemtype, const gchar *name, const gchar *info, gpointer value)
+skk_conf_item_new_with_value (SkkConfType type, const gchar *name, const gchar *info, gpointer value)
 {
 	SkkConfItem *ret;
 	ret = skk_conf_item_new ();
 	ret->type = type;
-	ret->itemtype = itemtype;
 	ret->name = name ? g_strdup (name) : NULL;
 	ret->info = info ? g_strdup (info) : NULL;
 	switch (type) {
@@ -920,32 +616,18 @@ skk_conf_item_destroy (SkkConfItem *item)
 }
 
 gboolean
-skk_conf_add_rule_listener (SkkConf *conf, SkkConfAddRuleListener listener, gpointer user_data)
+skk_conf_add_callback (SkkConf *conf, const gchar *key, SkkConfCallback callback, gpointer user_data)
 {
-	SkkConfAddRuleFunc *func;
+	SkkConfCallbackPrivate *p;
 	if (!conf)
 		return FALSE;
-	if (!listener)
+	if (!key || !callback)
 		return FALSE;
-	func = g_new0 (SkkConfAddRuleFunc, 1);
-	func->listener = listener;
-	func->user_data = user_data;
-	conf->conf_add_rule_listener = g_slist_append (conf->conf_add_rule_listener, func);
-	return TRUE;
-}
-
-gboolean
-skk_conf_add_rule_delete_listener (SkkConf *conf, SkkConfDeleteRuleListener listener, gpointer user_data)
-{
-	SkkConfDeleteRuleFunc *func;
-	if (!conf)
-		return FALSE;
-	if (!listener)
-		return FALSE;
-	func = g_new0 (SkkConfDeleteRuleFunc, 1);
-	func->listener = listener;
-	func->user_data = user_data;
-	conf->conf_delete_rule_listener = g_slist_append (conf->conf_delete_rule_listener, func);
+	p = g_new0 (SkkConfCallbackPrivate, 1);
+	p->key = g_strdup (key);
+	p->callback = callback;
+	p->user_data = user_data;
+	conf->callbacks = g_slist_append (conf->callbacks, p);
 	return TRUE;
 }
 
@@ -954,11 +636,27 @@ skk_conf_add_rule_delete_listener (SkkConf *conf, SkkConfDeleteRuleListener list
 #include "skkconv_kana.h"
 
 void
-listener (SkkConf *conf, const gchar *key, const gchar *hira, const gchar *kata, const gchar *append, gpointer user_data)
+callback (SkkConf *conf, GSList *value, const gchar *type, gpointer user_data)
 {
-	SkkConvRule *rule = (SkkConvRule *) user_data;
-	printf ("add_rule_listener\n");
-	printf ("key %s hira %s kata %s append %s\n", key, hira, kata, append);
+	SkkConvRule *rule = (SkkConvRule *)user_data;
+	gchar *key = NULL;
+	gchar *hira = NULL;
+	gchar *kata = NULL;
+	gchar *append = NULL;
+	SkkConfValue *v;
+	GSList *l;
+	for (l = value; l ; l = g_slist_next (l)) {
+		v = (SkkConfValue *)l->data;
+		if (!strcmp (v->name, "key")) {
+			key = v->value;
+		} else if (!strcmp (v->name, "hiragana")) {
+			hira = v->value;
+		} else if (!strcmp (v->name, "katakana")) {
+			kata = v->value;
+		} else if (!strcmp (v->name, "append")) {
+			append = v->value;
+		}
+	}
 	skk_conv_add_rule (rule, skk_conv_rule_item_new_with_value (key, hira, kata, append));
 	return;
 }
@@ -968,27 +666,31 @@ main (void)
 {
 	SkkConf *conf;
 	SkkConvRule *rule;
+	gint i,j;
 	conf = skk_conf_new();
 	rule = skk_conv_rule_new ();
-	printf("'%s' -> '%s'\n",".", skk_conv_get_hiragana(rule, ".", NULL));
-	skk_conf_add_rule_listener (conf, listener, rule);
-	skk_conf_load_rc(conf, NULL);
-	printf("skk_egg_like_newline = %s\n",
-					skk_conf_get_bool (conf, SKKCONF_BOOL_EGG_LIKE_NEWLINE) ? "TRUE" : "FALSE");
-	printf("skk_egg_like_newline = %s\n",
-					skk_conf_get_bool_by_name (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
-	printf("'%s' -> '%s'\n",".", skk_conv_get_hiragana(rule, ".", NULL));
-	skk_conf_set_bool (conf, SKKCONF_BOOL_EGG_LIKE_NEWLINE, FALSE);
-	printf("skk_egg_like_newline = %s\n",
-					skk_conf_get_bool_by_name (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
-	skk_conf_set_bool_by_name (conf, "skk-egg-like-newline", TRUE);
-	printf("skk_egg_like_newline = %s\n",
-					skk_conf_get_bool_by_name (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
-	printf ("skk_server_host = %s\n",
-			skk_conf_get_string_by_name (conf, "skk-server-host") ? 
-			skk_conf_get_string_by_name (conf, "skk-server-host") : "None");
-	printf("skk_plugin_use_dict = %s\n",
-					skk_conf_get_bool_by_name (conf, "skk-plugin-use-dict") ? "TRUE" : "FALSE");
+
+	if (getenv("SKK_PROF"))
+		j = 1000;
+	else
+		j = 1;
+
+	skk_conf_add_callback (conf, "/iiimf-skk/rule", callback, rule);
+	for (i = 0; i < j; i++) {
+		printf("'%s' -> '%s'\n",".", skk_conv_get_hiragana(rule, ".", NULL));
+		skk_conf_load_rc(conf, NULL);
+		printf("skk_egg_like_newline = %s\n",
+						skk_conf_get_bool (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
+		printf("'%s' -> '%s'\n",".", skk_conv_get_hiragana(rule, ".", NULL));
+		skk_conf_set_bool (conf, "skk-egg-like-newline", FALSE);
+		printf("skk_egg_like_newline = %s\n",
+						skk_conf_get_bool (conf, "skk-egg-like-newline") ? "TRUE" : "FALSE");
+		printf ("skk_server_host = %s\n",
+				skk_conf_get_string (conf, "skk-server-host") ? 
+				skk_conf_get_string (conf, "skk-server-host") : "None");
+		printf("skk_plugin_use_dict = %s\n",
+						skk_conf_get_bool (conf, "skk-plugin-use-dict") ? "TRUE" : "FALSE");
+	}
 	return 0;
 }
 #endif
