@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkmode.c,v 1.8 2004/04/23 07:20:57 famao Exp $ */
+/* $Id: skkmode.c,v 1.4.2.8 2003/03/08 12:35:31 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -46,25 +46,22 @@
 static void clear (SkkMode *mode);
 
 /* Get Status String */
-static gchar* get_status_string (SkkMode *mode);
+static gchar* get_mode_string (SkkMode *mode);
 
 /* Set Status */
-static void set_status (SkkMode *mode, SkkStatus status);
-static void set_j_status (SkkMode *mode, SkkJStatus jstatus);
-static void set_query_status (SkkMode *mode, SkkQueryStatus status);
+static void set_mode (SkkMode *mode, SkkMajorMode status);
+static void set_query_mode (SkkMode *mode, SkkQueryMode status);
 
 /* Get Status */
-static SkkStatus get_status (SkkMode *mode);
-static SkkJStatus get_j_status (SkkMode *mode);
-static SkkQueryStatus get_query_status (SkkMode *mode);
+static SkkMajorMode get_mode (SkkMode *mode);
+static SkkQueryMode get_query_mode (SkkMode *mode);
 
 /* Prepare prefix, postfix */
 static void prepare_mark (SkkMode *mode);
 
 /* Event */
 static void status_emit (SkkMode *mode);
-static void j_status_emit (SkkMode *mode);
-static void query_status_emit (SkkMode *mode);
+static void query_mode_emit (SkkMode *mode);
 
 
 typedef struct _SkkModeFunc SkkModeFunc;
@@ -81,42 +78,26 @@ status_emit (SkkMode *mode)
 	SkkModeFunc *func;
 	if (!mode)
 		return;
-	for (tmp_list = mode->status_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
+	for (tmp_list = mode->mode_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkModeFunc *)tmp_list->data;
 		if (func) {
-			func->listener (mode, skk_mode_get_status (mode), func->user_data);
+			func->listener (mode, skk_mode_get_mode (mode), func->user_data);
 		}
 	}
 	return;
 }
 
 static void
-j_status_emit (SkkMode *mode)
+query_mode_emit (SkkMode *mode)
 {
 	GSList *tmp_list;
 	SkkModeFunc *func;
 	if (!mode)
 		return;
-	for (tmp_list = mode->j_status_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
-		func = (SkkModeFunc *)tmp_list->data;
-		if (func) {
-			func->listener (mode, skk_mode_get_j_status (mode), func->user_data);
-		}
-	}
-	return;
-}
-
-static void
-query_status_emit (SkkMode *mode)
-{
-	GSList *tmp_list;
-	SkkModeFunc *func;
-	if (!mode)
-		return;
-	for (tmp_list = mode->query_status_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
+	for (tmp_list = mode->query_mode_listener; tmp_list; tmp_list = g_slist_next (tmp_list)) {
 		func = (SkkModeFunc *)tmp_list->data;
 		if (func)
-			func->listener (mode, skk_mode_get_query_status (mode), func->user_data);
+			func->listener (mode, skk_mode_get_query_mode (mode), func->user_data);
 	}
 	return;
 }
@@ -126,8 +107,8 @@ prepare_mark (SkkMode *mode)
 {
 	DEBUG_DO (printf ("skkmode.c : Entering prepare_mark\n"));
 	if (!mode) return;
-	switch (mode->query_status) {
-		case QUERY_NONE:
+	switch (mode->query_mode) {
+		case SKK_QUERY_NONE:
 			if (mode->query_prefix) {
 				g_free (mode->query_prefix);
 				mode->query_prefix = NULL;
@@ -137,7 +118,7 @@ prepare_mark (SkkMode *mode)
 				mode->query_suffix = NULL;
 			}
 			break;
-		case QUERY_NORMAL:
+		case SKK_QUERY_BEGIN:
 			if (mode->query_prefix) {
 				g_free (mode->query_prefix);
 			}
@@ -147,7 +128,7 @@ prepare_mark (SkkMode *mode)
 				mode->query_suffix = NULL;
 			}
 			break;
-		case QUERY_OKURI:
+		case SKK_QUERY_OKURI:
 			if (mode->query_prefix) {
 				g_free (mode->query_prefix);
 			}
@@ -157,7 +138,7 @@ prepare_mark (SkkMode *mode)
 			}
 			mode->query_suffix = g_strdup ("*");
 			break;
-		case QUERY_DONE:
+		case SKK_QUERY_DONE:
 			if (mode->query_prefix) {
 				g_free (mode->query_prefix);
 			}
@@ -174,12 +155,12 @@ prepare_mark (SkkMode *mode)
 }
 
 static gchar *
-get_status_string (SkkMode *mode)
+get_mode_string (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering get_status_string\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering get_mode_string\n"));
 	if (!mode) return NULL;
 	if (mode->status_buf) g_free (mode->status_buf);
-	switch (mode->status)
+	switch (mode->mode)
 	{
 		case SKK_J_MODE:
 			if (mode->jisx0201_kana) {
@@ -226,88 +207,54 @@ clear (SkkMode *mode)
 		mode->status_buf = NULL;
 	}
 	// mode->katakana = katakana;
-	if (mode->status != SKK_J_MODE)
-		skk_mode_set_status (mode, SKK_J_MODE);
-	if (mode->jstatus != SKK_NONE)
-		skk_mode_set_j_status (mode, SKK_NONE);
-	if (mode->query_status != QUERY_NONE)
-		skk_mode_set_query_status (mode, QUERY_NONE);
+	if (mode->mode != SKK_J_MODE)
+		skk_mode_set_mode (mode, SKK_J_MODE);
+	if (mode->query_mode != SKK_QUERY_NONE)
+		skk_mode_set_query_mode (mode, SKK_QUERY_NONE);
 	return;
 }
 
 static void
-set_status (SkkMode *mode, SkkStatus status)
+set_mode (SkkMode *mode, SkkMajorMode status)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering set_status\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering set_mode\n"));
 	if (!mode) return;
-	mode->status = status;
+	mode->mode = status;
 	skk_mode_prepare_mark (mode);
 	if (status == SKK_ABBREV_MODE) {
-		skk_mode_set_query_status (mode, QUERY_NORMAL);
+		skk_mode_set_query_mode (mode, SKK_QUERY_BEGIN);
 	}
 	status_emit (mode);
 	return;
 }
 
 static void
-set_query_status (SkkMode *mode, SkkQueryStatus status)
+set_query_mode (SkkMode *mode, SkkQueryMode status)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering set_query_status\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering set_query_mode\n"));
 	if (!mode)
 		return;
-	mode->query_status = status;
+	mode->query_mode = status;
 	skk_mode_prepare_mark (mode);
-	query_status_emit (mode);
+	query_mode_emit (mode);
 	return;
 }	
 
-static void
-set_j_status (SkkMode *mode, SkkJStatus jstatus)
+static SkkQueryMode
+get_query_mode (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering set_j_status\n"));
-	if (!mode) return;
-	mode->jstatus = jstatus;
-	switch (jstatus) {
-		case SKK_NONE:
-			skk_mode_set_query_status (mode, QUERY_NONE);
-			break;
-		case SKK_HENKAN:
-			skk_mode_set_query_status (mode, QUERY_NORMAL);
-			break;
-		case SKK_OKURI:
-			skk_mode_set_query_status (mode, QUERY_OKURI);
-			break;
-		case SKK_CHOICE:
-			skk_mode_set_query_status (mode, QUERY_DONE);
-			break;
-	}
-	j_status_emit (mode);
-	return;
-}
-
-static SkkQueryStatus
-get_query_status (SkkMode *mode)
-{
-	DEBUG_DO (printf ("skkmode.c : Entering get_query_status\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering get_query_mode\n"));
 	if (!mode)
-		return QUERY_NONE;
-	return mode->query_status;
+		return SKK_QUERY_NONE;
+	return mode->query_mode;
 }
 
-static SkkJStatus
-get_j_status (SkkMode *mode)
+static SkkMajorMode
+get_mode (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering get_j_status\n"));
-	if (!mode) return SKK_NONE;
-	return mode->jstatus;
-}
-
-static SkkStatus
-get_status (SkkMode *mode)
-{
-	DEBUG_DO (printf ("skkmode.c : Entering get_status\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering get_mode\n"));
 	if (!mode) return SKK_J_MODE;
-	return mode->status;
+	return mode->mode;
 }
 
 SkkMode *
@@ -317,9 +264,8 @@ skk_mode_new (void)
 	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_new\n"));
 	ret = g_new (SkkMode,1);
 	memset (ret,0,sizeof (SkkMode));
-	ret->status = SKK_J_MODE;
-	ret->query_status = QUERY_NONE;
-	ret->jstatus = SKK_NONE;
+	ret->mode = SKK_J_MODE;
+	ret->query_mode = SKK_QUERY_NONE;
 	ret->latin_str = g_strdup ("SKK");
 	ret->hirakana_str = g_strdup ("");
 	ret->katakana_str = g_strdup ("");
@@ -327,13 +273,11 @@ skk_mode_new (void)
 	ret->jisx0201_kana_str = g_strdup ("Ⱦ");
 	ret->abbrev_str = g_strdup ("a");
 	ret->clear = clear;
-	ret->set_status = set_status;
-	ret->set_j_status = set_j_status;
-	ret->set_query_status = set_query_status;
-	ret->get_status = get_status;
-	ret->get_status_string = get_status_string;
-	ret->get_query_status = get_query_status;
-	ret->get_j_status = get_j_status;
+	ret->set_mode = set_mode;
+	ret->set_query_mode = set_query_mode;
+	ret->get_mode = get_mode;
+	ret->get_mode_string = get_mode_string;
+	ret->get_query_mode = get_query_mode;
 	ret->prepare_mark = prepare_mark;
 	return ret;
 }
@@ -344,17 +288,13 @@ skk_mode_destroy (SkkMode *mode)
 	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_destroy\n"));
 	if (!mode) return;
 	skk_mode_clear (mode);
-	if (mode->status_listener) {
-		skk_utils_slist_free (mode->status_listener, TRUE, NULL, NULL);
-		mode->status_listener = NULL;
+	if (mode->mode_listener) {
+		skk_utils_slist_free (mode->mode_listener, TRUE, NULL, NULL);
+		mode->mode_listener = NULL;
 	}
-	if (mode->j_status_listener) {
-		skk_utils_slist_free (mode->j_status_listener, TRUE, NULL, NULL);
-		mode->j_status_listener = NULL;
-	}
-	if (mode->query_status_listener) {
-		skk_utils_slist_free (mode->query_status_listener, TRUE, NULL, NULL);
-		mode->query_status_listener = NULL;
+	if (mode->query_mode_listener) {
+		skk_utils_slist_free (mode->query_mode_listener, TRUE, NULL, NULL);
+		mode->query_mode_listener = NULL;
 	}
 	if (mode->latin_str)
 		g_free (mode->latin_str);
@@ -373,12 +313,12 @@ skk_mode_destroy (SkkMode *mode)
 }
 
 void
-skk_mode_set_status (SkkMode *mode,SkkStatus status)
+skk_mode_set_mode (SkkMode *mode,SkkMajorMode status)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_set_status\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_set_mode\n"));
 	if (!mode) return;
-	if (mode->set_status) {
-		mode->set_status (mode,status);
+	if (mode->set_mode) {
+		mode->set_mode (mode,status);
 	}
 	return;
 }
@@ -423,21 +363,21 @@ skk_mode_get_katakana (SkkMode *mode)
 
 
 void
-skk_mode_set_query_status (SkkMode *mode, SkkQueryStatus status)
+skk_mode_set_query_mode (SkkMode *mode, SkkQueryMode status)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_set_query_status\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_set_query_mode\n"));
 	if (!mode)
 		return;
-	if (mode->set_query_status) {
-		mode->set_query_status (mode, status);
+	if (mode->set_query_mode) {
+		mode->set_query_mode (mode, status);
 	}
 	return;
 }
 
 void
-skk_mode_get_query_status_strings (SkkMode *mode, const gchar **prefix, const gchar **append)
+skk_mode_get_query_mode_strings (SkkMode *mode, const gchar **prefix, const gchar **append)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_status_strings\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_mode_strings\n"));
 	if (!mode) return;
 	if (prefix)
 		*prefix = mode->query_prefix;
@@ -446,65 +386,42 @@ skk_mode_get_query_status_strings (SkkMode *mode, const gchar **prefix, const gc
 	return;
 }
 
-SkkQueryStatus
-skk_mode_get_query_status (SkkMode *mode)
+SkkQueryMode
+skk_mode_get_query_mode (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_status\n"));
-	if (!mode) return QUERY_NONE;
-	if (mode->get_query_status) {
-		return mode->get_query_status (mode);
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_mode\n"));
+	if (!mode) return SKK_QUERY_NONE;
+	if (mode->get_query_mode) {
+		return mode->get_query_mode (mode);
 	} else {
-		return QUERY_NONE;
+		return SKK_QUERY_NONE;
 	}
 }
 
 const gchar*
-skk_mode_get_query_status_prefix (SkkMode *mode)
+skk_mode_get_query_mode_prefix (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_status_prefix\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_mode_prefix\n"));
 	if (!mode) return NULL;
 	return mode->query_prefix;
 }
 
 const gchar *
-skk_mode_get_query_status_append (SkkMode *mode)
+skk_mode_get_query_mode_append (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_status_append\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_query_mode_append\n"));
 	if (!mode) return NULL;
 	return mode->query_suffix;
 }
 		
 
-SkkJStatus
-skk_mode_get_j_status (SkkMode *mode)
+SkkMajorMode
+skk_mode_get_mode (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_j_status\n"));
-	if (!mode) return SKK_NONE;
-	if (mode->get_j_status) {
-		return mode->get_j_status (mode);
-	} else {
-		return SKK_NONE;
-	}
-}
-
-void
-skk_mode_set_j_status (SkkMode *mode, SkkJStatus jstatus)
-{
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_set_j_status\n"));
-	if (!mode) return;
-	if (mode->set_j_status) {
-		mode->set_j_status (mode,jstatus);
-	}
-	return;
-}
-
-SkkStatus
-skk_mode_get_status (SkkMode *mode)
-{
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_status\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_mode\n"));
 	if (!mode) return SKK_J_MODE;
-	if (mode->get_status) {
-		return mode->get_status (mode);
+	if (mode->get_mode) {
+		return mode->get_mode (mode);
 	} else {
 		return SKK_J_MODE;
 	}
@@ -521,13 +438,13 @@ skk_mode_clear (SkkMode *mode)
 }
 
 gchar*
-skk_mode_get_status_string (SkkMode *mode)
+skk_mode_get_mode_string (SkkMode *mode)
 {
-	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_status_string\n"));
+	DEBUG_DO (printf ("skkmode.c : Entering skk_mode_get_mode_string\n"));
 	if (!mode) 
 		return g_strdup ("Unknown");
-	if (mode->get_status_string) {
-		return mode->get_status_string (mode);
+	if (mode->get_mode_string) {
+		return mode->get_mode_string (mode);
 	} else {
 		return g_strdup ("Unknown");
 	}
@@ -545,7 +462,7 @@ skk_mode_prepare_mark (SkkMode *mode)
 }
 
 void
-skk_mode_add_status_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data)
+skk_mode_add_mode_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data)
 {
 	SkkModeFunc *func;
 	if (!mode)
@@ -555,11 +472,11 @@ skk_mode_add_status_listener (SkkMode *mode, SkkModeListener listener, gpointer
 	func = g_new (SkkModeFunc, 1);
 	func->listener =listener;
 	func->user_data = user_data;
-	mode->status_listener = g_slist_append (mode->status_listener, func);
+	mode->mode_listener = g_slist_append (mode->mode_listener, func);
 }
 
 void
-skk_mode_add_j_status_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data)
+skk_mode_add_query_mode_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data)
 {
 	SkkModeFunc *func;
 	if (!mode)
@@ -569,22 +486,7 @@ skk_mode_add_j_status_listener (SkkMode *mode, SkkModeListener listener, gpointe
 	func = g_new (SkkModeFunc, 1);
 	func->listener = listener;
 	func->user_data = user_data;
-	mode->j_status_listener = g_slist_append (mode->j_status_listener, func);
-	return;
-}
-
-void
-skk_mode_add_query_status_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data)
-{
-	SkkModeFunc *func;
-	if (!mode)
-		return;
-	if (!listener)
-		return;
-	func = g_new (SkkModeFunc, 1);
-	func->listener = listener;
-	func->user_data = user_data;
-	mode->query_status_listener = g_slist_append (mode->query_status_listener, func);
+	mode->query_mode_listener = g_slist_append (mode->query_mode_listener, func);
 	return;
 }
 
@@ -685,41 +587,29 @@ skk_mode_set_abbrev_str (SkkMode *mode, const gchar *str)
 }
 
 gboolean
-skk_mode_remove_status_listener (SkkMode *mode, SkkModeListener listener)
+skk_mode_remove_mode_listener (SkkMode *mode, SkkModeListener listener)
 {
 	return FALSE;
 }
 
 gboolean
-skk_mode_remove_j_status_listener (SkkMode *mode, SkkModeListener listener)
-{
-	return FALSE;
-}
-
-gboolean
-skk_mode_remove_query_status_listener (SkkMode *mode, SkkModeListener listener)
+skk_mode_remove_query_mode_listener (SkkMode *mode, SkkModeListener listener)
 {
 	return FALSE;
 }	
 
 #ifdef SKKMODE_DEBUG
 static void
-status_listener (SkkMode *mode, gint status, gpointer user_data)
+mode_listener (SkkMode *mode, gint status, gpointer user_data)
 {
 	g_message ("status changed to %d", status);
-	g_message ("mode string is %s", skk_mode_get_status_string (mode));
+	g_message ("mode string is %s", skk_mode_get_mode_string (mode));
 }
 
 static void
-j_status_listener (SkkMode *mode, gint status, gpointer user_data)
+query_mode_listener (SkkMode *mode, gint status, gpointer user_data)
 {
-	g_message ("j_status changed to %d", status);
-}
-
-static void
-query_status_listener (SkkMode *mode, gint status, gpointer user_data)
-{
-	g_message ("query_status changed to %d", status);
+	g_message ("query_mode changed to %d", status);
 }
 
 int
@@ -728,12 +618,10 @@ main (void)
 	SkkMode *mode;
 	mode = skk_mode_new ();
 	/* add listener */
-	skk_mode_add_status_listener (mode, status_listener, NULL);
-	skk_mode_add_j_status_listener (mode, j_status_listener, NULL);
-	skk_mode_add_query_status_listener (mode, query_status_listener, NULL);
-	skk_mode_set_status (mode, SKK_J_MODE);
-	skk_mode_set_j_status (mode, SKK_CHOICE);
-	skk_mode_set_query_status (mode, QUERY_DONE);
+	skk_mode_add_mode_listener (mode, mode_listener, NULL);
+	skk_mode_add_query_mode_listener (mode, query_mode_listener, NULL);
+	skk_mode_set_mode (mode, SKK_J_MODE);
+	skk_mode_set_query_mode (mode, SKK_QUERY_DONE);
 	return 0;
 }
 #endif
