@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkmode.h,v 1.4 2003/02/19 17:44:52 famao Exp $ */
+/* $Id: skkmode.h,v 1.2.2.7 2003/02/22 01:27:45 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -59,24 +59,20 @@ struct _SkkMode
 	gchar *abbrev_str;
 	gboolean katakana;
 	gboolean jisx0201_kana;
-	SkkStatus status;
-	SkkJStatus jstatus;
-	SkkQueryStatus query_status;
+	SkkMajorMode mode;
+	SkkQueryMode query_mode;
 
-	GSList *status_listener;
-	GSList *j_status_listener;
-	GSList *query_status_listener;
+	GSList *mode_listener;
+	GSList *query_mode_listener;
 
 	/* Function */
 	void (*clear)(SkkMode *mode);
-	void (*set_status)(SkkMode *mode,SkkStatus status);
-	void (*set_j_status) (SkkMode *mode, SkkJStatus jstatus);
-	void (*set_query_status) (SkkMode *mode, SkkQueryStatus status);
+	void (*set_mode)(SkkMode *mode,SkkMajorMode status);
+	void (*set_query_mode) (SkkMode *mode, SkkQueryMode status);
 	void (*prepare_mark) (SkkMode *mode);
-	gchar* (*get_status_string) (SkkMode *mode);
-	SkkStatus (*get_status) (SkkMode *mode);
-	SkkJStatus (*get_j_status) (SkkMode *mode);
-	SkkQueryStatus (*get_query_status) (SkkMode *mode);
+	gchar* (*get_mode_string) (SkkMode *mode);
+	SkkMajorMode (*get_mode) (SkkMode *mode);
+	SkkQueryMode (*get_query_mode) (SkkMode *mode);
 };
 
 
@@ -84,9 +80,9 @@ SkkMode * skk_mode_new (void);
 void skk_mode_destroy (SkkMode *mode);
 void skk_mode_clear (SkkMode *mode);
 
-void skk_mode_set_status (SkkMode *mode, SkkStatus status);
-gchar * skk_mode_get_status_string (SkkMode *mode);
-SkkStatus skk_mode_get_status (SkkMode *mode);
+void skk_mode_set_mode (SkkMode *mode, SkkMajorMode status);
+gchar * skk_mode_get_mode_string (SkkMode *mode);
+SkkMajorMode skk_mode_get_mode (SkkMode *mode);
 
 void skk_mode_set_katakana (SkkMode *mode,gboolean val);
 gboolean skk_mode_get_katakana (SkkMode *mode);
@@ -96,22 +92,18 @@ gboolean skk_mode_get_jisx0201_katakana (SkkMode *mode);
 
 void skk_mode_prepare_mark (SkkMode *mode);
 
-void skk_mode_set_query_status (SkkMode *mode, SkkQueryStatus status);
-void skk_mode_get_query_status_strings (SkkMode *mode, const gchar **prefix, const gchar** append);
-const gchar* skk_mode_get_query_status_prefix (SkkMode *mode);
-const gchar* skk_mode_get_query_status_append (SkkMode *mode);
-SkkQueryStatus skk_mode_get_query_status (SkkMode *mode);
+void skk_mode_set_query_mode (SkkMode *mode, SkkQueryMode status);
+void skk_mode_get_query_mode_strings (SkkMode *mode, const gchar **prefix, const gchar** append);
+const gchar* skk_mode_get_query_mode_prefix (SkkMode *mode);
+const gchar* skk_mode_get_query_mode_append (SkkMode *mode);
+SkkQueryMode skk_mode_get_query_mode (SkkMode *mode);
 
-void skk_mode_set_j_status (SkkMode *mode,SkkJStatus jstatus);
-SkkJStatus skk_mode_get_j_status (SkkMode *mode);
 
-void skk_mode_add_status_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data);
-void skk_mode_add_j_status_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data);
-void skk_mode_add_query_status_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data);
+void skk_mode_add_mode_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data);
+void skk_mode_add_query_mode_listener (SkkMode *mode, SkkModeListener listener, gpointer user_data);
 
-gboolean skk_mode_remove_status_listener (SkkMode *mode, SkkModeListener listener);
-gboolean skk_mode_remove_j_status_listener (SkkMode *mode, SkkModeListener listener);
-gboolean skk_mode_remove_query_status_listener (SkkMode *mode, SkkModeListener listener);
+gboolean skk_mode_remove_mode_listener (SkkMode *mode, SkkModeListener listener);
+gboolean skk_mode_remove_query_mode_listener (SkkMode *mode, SkkModeListener listener);
 
 void skk_mode_set_latin_str (SkkMode *mode, const gchar *str);
 void skk_mode_set_hirakana_str (SkkMode *mode, const gchar *str);
@@ -119,4 +111,5 @@ void skk_mode_set_katakana_str (SkkMode *mode, const gchar *str);
 void skk_mode_set_jisx0208_latin_str (SkkMode *mode, const gchar *str);
 void skk_mode_set_jisx0201_kana_str (SkkMode *mode, const gchar *str);
 void skk_mode_set_abbrev_str (SkkMode *mode, const gchar *str);
+
 #endif /* HAVE_SKKCONV_MODE_H */
