@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkpoboximpl.c,v 1.4 2004/04/23 07:20:57 famao Exp $ */
+/* $Id: skkpoboximpl.c,v 1.1.2.11 2003/03/08 12:36:48 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -73,8 +73,8 @@ static SkkQueryFunctionImpl *get_new (SkkQuery *query)
 		impl->hostname = NULL;
 		impl->port = NULL;
 	} else {
-		impl->hostname = skk_query_get_conf_string_by_name (query, "skk-pobox-server-host");
-		impl->port = skk_query_get_conf_string_by_name (query, "skk-pobox-server-port");
+		impl->hostname = skk_query_get_conf_string (query, "skk-pobox-server-host");
+		impl->port = skk_query_get_conf_string (query, "skk-pobox-server-port");
 	}
 	return (SkkQueryFunctionImpl*) impl;
 }
@@ -129,6 +129,7 @@ retry:
 		if (list_count && *list_count == '\0') {
 			continue;
 		}
+		DEBUG_DO(g_message ("%s",list_count));
 		item = g_strsplit (list_count, "\t", strlen (list_count));
 		for (j = 0, item_count = item[0]; item_count; j++, item_count = item[j]) {
 			if (item_count && *item_count == '\0') {
