@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skkquery.c,v 1.9 2003/02/19 17:44:52 famao Exp $ */
+/* $Id: skkquery.c,v 1.2.2.17 2003/04/17 06:19:23 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -391,31 +391,31 @@ skk_query_set_conf (SkkQuery *query, SkkConf *conf)
 }
 
 gchar *
-skk_query_get_conf_string_by_name (SkkQuery *query, const gchar *name)
+skk_query_get_conf_string (SkkQuery *query, const gchar *name)
 {
 	if (!query)
 		return NULL;
 	if (!query->conf)
 		return NULL;
-	return skk_conf_get_string_by_name (query->conf, name);
+	return skk_conf_get_string (query->conf, name);
 }
 
 gint
-skk_query_get_conf_num_by_name (SkkQuery *query, const gchar *name)
+skk_query_get_conf_num (SkkQuery *query, const gchar *name)
 {
 	if (!query)
 		return -1;
 	if (!query->conf)
 		return -1;
-	return skk_conf_get_num_by_name (query->conf, name);
+	return skk_conf_get_num (query->conf, name);
 }
 
 gboolean
-skk_query_get_conf_bool_by_name (SkkQuery *query, const gchar *name)
+skk_query_get_conf_bool (SkkQuery *query, const gchar *name)
 {
 	if (!query)
 		return FALSE;
 	if (!query->conf)
 		return FALSE;
-	return skk_conf_get_bool_by_name (query->conf, name);
+	return skk_conf_get_bool (query->conf, name);
 }
