@@ -32,7 +32,7 @@
  *
  */
 
-/* $Id: skktypes.h,v 1.10 2003/02/19 17:44:52 famao Exp $ */
+/* $Id: skktypes.h,v 1.4.2.23 2003/04/14 23:15:23 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -41,19 +41,22 @@
 #define HAVE_SKKTYPES_H
 
 /* skkmode */
-typedef enum {
-	SKK_NONE   = 1 << 0,
-	SKK_HENKAN = 1 << 1,
-	SKK_OKURI  = 1 << 2,
-	SKK_CHOICE = 1 << 3
-}SkkJStatus;
 
 typedef enum {
 	SKK_J_MODE              = 1 << 0,
 	SKK_LATIN_MODE          = 1 << 1,
 	SKK_JISX0208_LATIN_MODE = 1 << 2,
-	SKK_ABBREV_MODE         = 1 << 3
-}SkkStatus;
+	SKK_ABBREV_MODE         = 1 << 3,
+	SKK_ALL_MODE            = 0xffff
+}SkkMajorMode;
+
+typedef enum {
+	SKK_QUERY_NONE   = 1 << 0,
+	SKK_QUERY_BEGIN  = 1 << 1,
+	SKK_QUERY_OKURI  = 1 << 2,
+	SKK_QUERY_DONE   = 1 << 3,
+	SKK_QUERY_ALL    = 0xffff
+}SkkQueryMode;
 
 /* must sync with SkkStatus */
 typedef enum {
@@ -64,13 +67,6 @@ typedef enum {
 	SKK_FUNC_ALL            = 0xffff
 }SkkFunctionType;
 
-typedef enum {
-	QUERY_NONE   = 1 << 0,
-	QUERY_NORMAL = 1 << 1,
-	QUERY_OKURI  = 1 << 2,
-	QUERY_DONE   = 1 << 3,
-	QUERY_ALL    = 0xffff
-}SkkQueryStatus;
 
 /* skkcomm */
 typedef enum {
@@ -131,37 +127,6 @@ typedef enum {
 	SKKCONF_TYPE_INVALID
 }SkkConfType;
 
-typedef enum {
-	SKKCONF_ITEM_INVALID = 0,
-	/* Bool */
-	SKKCONF_BOOL_DABBREV_LIKE_COMPLETION,
-	SKKCONF_BOOL_DATE_AD,
-	SKKCONF_BOOL_DELETE_IMPLIES_KAKUTEI,
-	SKKCONF_BOOL_DELETE_OKURI_WHEN_QUIT,
-	SKKCONF_BOOL_ECHO,
-	SKKCONF_BOOL_EGG_LIKE_NEWLINE,
-	SKKCONF_BOOL_USE_CANNA,
-	SKKCONF_BOOL_USE_DICT,
-	SKKCONF_BOOL_USE_LDICT,
-	SKKCONF_BOOL_USE_LOOK,
-	SKKCONF_BOOL_USE_POBOX,
-	/* Num */
-	SKKCONF_NUM_CANDIDATE_COUNT,
-	/* String */
-	SKKCONF_STRING_JISYO,
-	SKKCONF_STRING_MODE_ABBREV,
-	SKKCONF_STRING_MODE_HIRAKANA,
-	SKKCONF_STRING_MODE_JISX0201_KANA,
-	SKKCONF_STRING_MODE_JISX0208_LATIN,
-	SKKCONF_STRING_MODE_KATAKANA,
-	SKKCONF_STRING_MODE_LATIN,
-	SKKCONF_STRING_OKURI_JISYO,
-	SKKCONF_STRING_POBOX_SERVER_HOST,
-	SKKCONF_STRING_POBOX_SERVER_PORT,
-	SKKCONF_STRING_SERVER_HOST,
-	SKKCONF_STRING_SERVER_PORT
-}SkkConfItemType;
-
 /* skkconv_kana.c */
 typedef enum {
 	SKKCONV_TYPE_NONE = 0,
@@ -171,29 +136,6 @@ typedef enum {
 	SKKCONV_TYPE_USER_FUNC
 }SkkConvRuleType;
 
-/* skkfunc.c */
-typedef enum {
-	SKKFUNC_TRANSLATE_BEGIN = 1,
-	SKKFUNC_BACKWARD_DELETE,
-	SKKFUNC_CANCEL,
-	SKKFUNC_CATEGORY_CHANGE,
-	SKKFUNC_CODETABLE,
-	SKKFUNC_COMPLETE_BEGIN,
-	SKKFUNC_COMPLETE_NEXT,
-	SKKFUNC_COMPLETE_PREV,
-	SKKFUNC_COMPLETE_TRANSLATE,
-	SKKFUNC_CURSOR_LEFT,
-	SKKFUNC_CURSOR_RIGHT,
-	SKKFUNC_HALFWIDTH_KANA_SWITCH,
-	SKKFUNC_KAKUTEI,
-	SKKFUNC_KANA_SWITCH,
-	SKKFUNC_MODE_CHANGE,
-	SKKFUNC_POSTFIX,
-	SKKFUNC_PREFIX,
-	SKKFUNC_TODAY,
-	SKKFUNC_UNDO
-}SkkFunction;
-
 /* skkel.c */
 
 typedef enum {
