@@ -34,7 +34,7 @@
  *
  */
 
-/* $Id: skk.c,v 1.24 2004/04/23 07:20:57 famao Exp $ */
+/* $Id: skk.c,v 1.9.2.45 2003/04/17 08:45:19 famao Exp $ */
 
 /* vi:set ts=4 sw=4: */
 
@@ -81,10 +81,6 @@ Sun Microsystems, Inc. or its licensors is granted.
 
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -99,8 +95,9 @@ Sun Microsystems, Inc. or its licensors is granted.
 #include "skkbuffer.h"
 #include "skkconv_kana.h"
 #include "skkconv_jisx0208_latin.h"
-#include "skkkeymap.h"
+#include "skkfunction.h"
 #include "skkkeysym.h"
+#include "skkkeysymutil.h"
 #include "skkutils.h"
 /*
  * header files for SunIM library
@@ -175,8 +172,12 @@ static void skk_lookup_listener (SkkBuffer *buf, gchar **label, gchar **candidat
 static void skk_codetable_listener (SkkBuffer *buf, gpointer user_data);
 static void skk_status_listener (SkkMode *mode, gint status, gpointer user_data);
 
-static void skk_add_rule_listener (SkkConf *conf, const gchar *key, const gchar *hira,
-		const gchar *kata, const gchar *append, gpointer user_data);
+static void skk_add_rule_callback (SkkConf *conf, GSList *value, const gchar *type,
+		gpointer user_data);
+static void skk_add_delete_rule_callback (SkkConf *conf, GSList *value, const gchar *type,
+		gpointer user_data);
+static void skk_add_functionmap_callback (SkkConf *conf, GSList *v, const gchar *type,
+		gpointer user_data);
 
 /* Preedit */
 static void skk_preedit_done (iml_session_t * s);
@@ -261,7 +262,8 @@ struct _SkkSessionData
 struct _SkkDesktopData
 {
 	int drop_priv;
-	SkkFunc *skkfunc;
+	SkkFunction *skkfunc;
+	SkkFunctionMap *skkfuncmap;
 	SkkQuery *skkquery;
 	SkkConf *skkconf;
 	SkkConvRule *skkconv;
@@ -467,7 +469,7 @@ if_GetIfInfo (IMArgList args, int num_args)
 				args->value = (IMArgVal) objects;
 				break;
 			case IF_NEED_THREAD_LOCK:
-				args->value = (IMArgVal) True;
+				args->value = (IMArgVal) False;
 				break;
 			default:
 				break;
@@ -586,11 +588,17 @@ if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 	}
 	d = g_new0 (SkkDesktopData, 1);
 	d->drop_priv = skk_drop_privilege (desktop->user_name);
-	d->skkfunc = skk_func_new ();
+	d->skkfuncmap = skk_function_map_new ();
+	d->skkfunc = skk_function_new (d->skkfuncmap);
 	d->skkquery = skk_query_new ();
 	d->skkconf = skk_conf_new ();
 	d->skkconv = skk_conv_rule_new ();
-	skk_conf_add_rule_listener (d->skkconf, skk_add_rule_listener, d->skkconv);
+	skk_conf_add_callback (d->skkconf, "/iiimf-skk/rule", skk_add_rule_callback, d->skkconv);
+	skk_conf_add_callback (d->skkconf, "/iiimf-skk/functionmap",
+			skk_add_functionmap_callback, d->skkfuncmap);
+	skk_conf_add_callback (d->skkconf, "/iiimf-skk/delete-rule",
+			skk_add_delete_rule_callback, d->skkconv);
+
 	if (d->drop_priv) {
 #ifdef DEBUG
 		printf ("Drop Privilege\n");
@@ -604,19 +612,19 @@ if_skk_OpenDesktop (iml_desktop_t * desktop, IMArgList args, int num_args)
 	skk_query_set_conf (d->skkquery, d->skkconf);
 
 	/* TODO configurable */
-	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-ldict")) {
+	if (skk_conf_get_bool (d->skkconf, "skk-plugin-use-ldict")) {
 		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_LOCAL));
 	}
-	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-dict")) {
+	if (skk_conf_get_bool (d->skkconf, "skk-plugin-use-dict")) {
 		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_SERVER));
 	}
-	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-canna")) {
+	if (skk_conf_get_bool (d->skkconf, "skk-plugin-use-canna")) {
 		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_CANNA_SERVER));
 	}
-	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-look")) {
+	if (skk_conf_get_bool (d->skkconf, "skk-plugin-use-look")) {
 		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_LOOK));
 	}
-	if (skk_conf_get_bool_by_name (d->skkconf, "skk-plugin-use-pobox")) {
+	if (skk_conf_get_bool (d->skkconf, "skk-plugin-use-pobox")) {
 		skk_query_add_item (d->skkquery, skk_query_item_new (d->skkquery, SKKQUERY_POBOX_SERVER));
 	}
 	desktop->specific_data = (void *)d;
@@ -658,7 +666,10 @@ if_skk_CloseDesktop (iml_desktop_t * desktop)
 		skk_query_destroy (d->skkquery);
 	}
 	if (d->skkfunc) {
-		skk_func_destroy (d->skkfunc);
+		skk_function_destroy (d->skkfunc);
+	}
+	if (d->skkfuncmap) {
+		skk_function_map_destroy (d->skkfuncmap);
 	}
 	if (d->skkconf) {
 		skk_conf_destroy (d->skkconf);
@@ -681,10 +692,93 @@ if_skk_CloseDesktop (iml_desktop_t * desktop)
 }
 
 static void
-skk_add_rule_listener (SkkConf *conf , const gchar *key,
-		const gchar *hira, const gchar *kata, const gchar *append, gpointer user_data)
+skk_add_functionmap_callback (SkkConf *conf, GSList *v, const gchar *type, gpointer user_data)
+{
+        SkkFunctionMap *map = (SkkFunctionMap *)user_data;
+        SkkConfValue *value;
+        GSList *l;
+        gchar *name = NULL;
+        guint mode = 0;
+        guint submode = 0;
+        guint key = 0;
+        guint mask = 0;
+        guint actual_key = 0;
+        if (!type)
+                return;
+        for (l = v; l; l = g_slist_next (l)) {
+                value = (SkkConfValue *)l->data;
+                if (!strcmp (value->name, "name")) {
+                        name = value->value;
+                } else if (!strcmp (value->name, "mode")) {
+                        mode = skk_keysym_keyname_to_mode (value->value);
+                } else if (!strcmp (value->name, "querymode")) {
+                        submode = skk_keysym_keyname_to_query_mode (value->value);
+                } else if (!strcmp (value->name, "key")) {
+                        if (!*(value->value))
+                                key = 0;
+                        else
+                                key = skk_keysym_keyname_to_keycode (value->value);
+                } else if (!strcmp (value->name, "mask")) {
+                        mask = skk_keysym_keyname_to_mask (value->value);
+                } else if (!strcmp (value->name, "actualkey")) {
+			if (!*(value->value))
+				actual_key = 0;
+			else
+	                        actual_key = skk_keysym_keyname_to_keycode (value->value);
+                }
+        }
+        if (!strcmp (type, "remove")) {
+                skk_function_map_remove_item (map,
+                                mode, submode, key, mask);
+        } else if (!strcmp (type, "add")) {
+                skk_function_map_add_item (map,
+                                skk_function_map_item_new (name,
+                                        mode, submode, key, mask, actual_key));
+        }
+        return;
+}
+
+static void
+skk_add_delete_rule_callback (SkkConf *conf, GSList *value, const gchar *type, gpointer user_data)
 {
 	SkkConvRule *rule = (SkkConvRule *)user_data;
+	SkkConfValue *v;
+	GSList *l;
+	for (l = value; l; l = g_slist_next (l)) {
+		v = (SkkConfValue *)l->data;
+		if (!strcmp (v->name, "key")) {
+			if (v->value && *(v->value))
+				skk_conv_delete_rule (rule, v->value);
+		}
+	}
+	return;
+}
+
+static void
+skk_add_rule_callback (SkkConf *conf, GSList *value, const gchar *type, gpointer user_data)
+{
+	SkkConvRule *rule = (SkkConvRule *)user_data;
+        gchar *key = NULL;
+        gchar *hira = NULL;
+        gchar *kata = NULL;
+        gchar *append = NULL;
+        SkkConfValue *v;
+        GSList *l;
+        for (l = value; l ; l = g_slist_next (l)) {
+                v = (SkkConfValue *)l->data;
+                if (!strcmp (v->name, "key")) {
+                        key = v->value;
+                } else if (!strcmp (v->name, "hiragana")) {
+			if (*(v->value))
+	                        hira = v->value;
+                } else if (!strcmp (v->name, "katakana")) {
+			if (*(v->value))
+	                        kata = v->value;
+                } else if (!strcmp (v->name, "append")) {
+			if (*(v->value))
+	                        append = v->value;
+                }
+        }
 #if DEBUG
 	printf ("add rule %s %s %s %s\n", key, hira, kata, append);
 #endif
@@ -696,9 +790,12 @@ static void
 skk_result_listener (SkkBuffer *buf, gchar *value, gpointer user_data)
 {
 	iml_session_t *s = (iml_session_t *) user_data;
+	if (skkbuf_in_destroy (buf))
+		return;
 	if (!value) {
 		return;
 	}
+	g_message ("commit string %s", value);
 	skk_commit_string (s, value);
 	return;
 }
@@ -707,18 +804,28 @@ static void
 skk_preedit_listener (SkkBuffer *buf, gchar *value, gpointer user_data)
 {
 	iml_session_t *s = (iml_session_t *) user_data;
-	if (!value)
+	if (skkbuf_in_destroy (buf))
+		return;
+	if (!value) {
+		g_message ("preedit_clear");
+#if 1
 		skk_preedit_clear (s);
-	else {
+#endif
+		g_message ("preedit_clear");
+	} else {
+		g_message ("preedit_draw %s", value);
 		skk_preedit_draw (s, value);
 	}
 	return;
 }
 
 static void
-skk_status_listener (SkkMode *buf, gint status, gpointer user_data)
+skk_status_listener (SkkMode *mode, gint status, gpointer user_data)
 {
 	iml_session_t *s = (iml_session_t *) user_data;
+	SkkBuffer *buf  = skk_session_buf (s);
+	if (skkbuf_in_destroy (buf))
+		return;
 	skk_status_draw (s);
 }
 
@@ -796,7 +903,7 @@ if_skk_CreateSC (iml_session_t * s, IMArgList args, int num_args)
 	SkkDesktopData *d = skk_desktop_data (s);
 	SkkSessionData *ses = g_new0 (SkkSessionData, 1);
 	SkkBuffer *buf = skk_buffer_new ();
-	skk_buffer_set_status (buf,SKK_LATIN_MODE);
+	skk_buffer_set_mode (buf,SKK_LATIN_MODE);
 	if (!ses)
 		return False;
 	ses->conversion_start = False;
@@ -816,8 +923,10 @@ if_skk_CreateSC (iml_session_t * s, IMArgList args, int num_args)
 	skk_buffer_set_rule (buf, d->skkconv);
 	skk_buffer_add_result_listener (buf, skk_result_listener, (gpointer) s);
 	skk_buffer_add_preedit_listener (buf, skk_preedit_listener, (gpointer) s);
-	skk_mode_add_status_listener (skkbuf_mode (buf), skk_status_listener, (gpointer) s);
-	skk_mode_add_j_status_listener (skkbuf_mode (buf), skk_status_listener, (gpointer) s);
+	skk_mode_add_mode_listener (skkbuf_mode (buf), skk_status_listener, (gpointer) s);
+#if 0
+	skk_mode_add_query_status_listener (skkbuf_mode (buf), skk_status_listener, (gpointer) s);
+#endif
 	skk_buffer_add_adddict_listener (buf, skk_adddict_listener, (gpointer) s);
 	skk_buffer_add_lookup_listener (buf, skk_lookup_listener, (gpointer) s);
 	skk_buffer_add_codetable_listener (buf, skk_codetable_listener, (gpointer) s);
@@ -830,6 +939,9 @@ if_skk_DestroySC (iml_session_t * s)
 	SkkSessionData *ses;
 	ses = skk_session_data (s);
 	skk_buffer_destroy (ses->skkbuf);
+	if (ses->preedit_start) {
+		skk_preedit_done (s);
+	}
 	g_free (ses);
 	return True;
 }
@@ -908,10 +1020,69 @@ if_skk_SetSCValues (iml_session_t * s, IMArgList args, int num_args)
 static IMText *
 if_skk_ResetSC (iml_session_t *s)
 {
-	iml_inst *lp;
-	lp = s->If->m->iml_make_preedit_erase_inst (s);
-	s->If->m->iml_execute (s, &lp);
-	return (IMText *) NULL;
+	SkkBuffer *buf = skk_session_buf (s);
+	SkkSessionData *sses = skk_session_data (s);
+	SkkCursor *cursor = skkbuf_cursor (buf);
+	gchar *preedit_string;
+	IMText *ret = NULL;
+	if (skk_buffer_has_preedit (buf)) {
+		preedit_string = skk_buffer_get_preedit_string (buf);
+		if (cursor->revLen) {
+			/* Create IMText with feedbacks */
+			int nb[3], fts[3];
+			unsigned char *strs[3];
+			nb[0] = cursor->revPos;
+			nb[1] = cursor->revLen;
+			nb[2] = strlen (preedit_string) - cursor->revPos - cursor->revLen;
+			fts[0] = SKK_Feedback_Input;
+			fts[1] = SKK_Feedback_Strong;
+			fts[2] = SKK_Feedback_Input;
+			strs[0] = preedit_string;
+			strs[1] = preedit_string + cursor->revPos;
+			strs[2] = preedit_string + cursor->revPos + cursor->revLen;
+			g_message ("%s %s %s %d %d %d", strs[0], strs[1], strs[2],
+					nb[0], nb[1], nb[2]);
+			if (nb[2] == 0) {
+				ret = skk_strtoimtext (s, 2, nb, strs, fts);
+			} else {
+				ret = skk_strtoimtext (s, 3, nb, strs, fts);
+			}
+		} else if (cursor->caret){
+			int nb[3], fts[3];
+			unsigned char *strs[3];
+			int cur = skk_utils_last_charbytes_nth_all (preedit_string, cursor->caret);
+			int charlen = skk_utils_strlen (preedit_string);
+			int cur_pos = skk_utils_charbytes_nth (preedit_string, charlen - cursor->caret - 1);
+			nb[0] = strlen (preedit_string) - cur;
+			nb[1] = cur_pos;
+			nb[2] = strlen (preedit_string) - nb[0] - nb[1];
+			fts[0] = SKK_Feedback_Input;
+			fts[1] = SKK_Feedback_Strong;
+			fts[2] = SKK_Feedback_Input;
+			strs[0] = preedit_string;
+			strs[1] = preedit_string + strlen (preedit_string) - cur;
+			strs[2] = preedit_string + nb[0] +  nb[1];
+			g_message ("caret %s %s %s %d %d %d", strs[0], strs[1], strs[2],
+					nb[0], nb[1], nb[2]);
+			if (nb[2] == 0) {
+				ret = skk_strtoimtext (s, 2, nb, strs, fts);
+			} else {
+				ret = skk_strtoimtext (s, 3, nb, strs, fts);
+			}
+		} else {
+			/* Create IMText with feedbacks */
+			int nb[1],fts[1];
+			unsigned char *strs[1];
+			nb[0] = strlen(preedit_string);
+			fts[0] = SKK_Feedback_Input;
+			strs[0] = preedit_string;
+			ret = skk_strtoimtext (s, 1, nb, strs, fts);
+		}
+		g_free (preedit_string);
+		return ret;
+	} else {
+		return (IMText *) NULL;
+	}
 }
 
 static void
@@ -974,7 +1145,7 @@ skk_make_conversion_on (iml_session_t *s)
 		ses->conversion_start = True;
 		skk_buffer_set_katakana (buf, FALSE);
 		skk_buffer_set_jisx0201_katakana (buf, FALSE);
-		skk_buffer_set_status (buf, SKK_J_MODE);
+		skk_buffer_set_mode (buf, SKK_J_MODE);
 	}
 	return;
 }
@@ -996,13 +1167,13 @@ static void
 skk_commit_imtext (iml_session_t *s, IMText *text)
 {
 	iml_inst *lp;
-#if 0
 	SkkSessionData *ses = skk_session_data (s);
-#endif
 	if (!text)
 		return;
 
+	/* erase preedit area first */
 #if 0
+	skk_preedit_clear (s);
 	/* erase preedit area first */
 	if (ses->preedit_start == True) {
 		skk_preedit_clear (s);
@@ -1012,11 +1183,7 @@ skk_commit_imtext (iml_session_t *s, IMText *text)
 	/* commit string */
 	lp = s->If->m->iml_make_commit_inst (s, text);
 	s->If->m->iml_execute (s, &lp);
-
-#if 0
-	/* BUG workaround */
 	skk_preedit_clear (s);
-#endif
 }
 
 static void
@@ -1069,7 +1236,7 @@ skk_translate_keyevent (iml_session_t *s, IMKeyListEvent * kev)
   IMKeyEventStruct *k = (IMKeyEventStruct *) kev->keylist;
   printf ("iml_session_t() keycode=%x,keychar=%x,state=%x\n",
 	  k->keyCode, k->keyChar, k->modifier);
-  printf ("translate_keyevent status %d\n",skk_buffer_get_status (buf));
+  printf ("translate_keyevent status %d\n",skk_buffer_get_mode (buf));
 #endif
   if (ses->dict_start && ses->dict_aux_ok) {
 	  return SKK_NOTHING;
@@ -1077,7 +1244,7 @@ skk_translate_keyevent (iml_session_t *s, IMKeyListEvent * kev)
   if (ses->luc_start == True) {
 	  return skk_translate_keyevent_luc (s, kev);
   }
-  switch (skk_buffer_get_status (buf)) {
+  switch (skk_buffer_get_mode (buf)) {
 	  case SKK_J_MODE:
 		  return skk_translate_keyevent_jmode (s, kev);
 	  case SKK_LATIN_MODE:
@@ -1310,7 +1477,7 @@ skk_process_keyevent_abbrev (iml_session_t *s, IMKeyListEvent *kev)
 	}
 	if (skk_buffer_can_clear (buf)) {
 		skk_buffer_clear (buf);
-		skk_buffer_set_status (buf, SKK_J_MODE); /* return to default */
+		skk_buffer_set_mode (buf, SKK_J_MODE); /* return to default */
 	}
 	return;
 }
@@ -1321,7 +1488,7 @@ skk_process_keyevent_jisx0208_latin (iml_session_t *s, IMKeyListEvent *kev)
 	SkkBuffer *buf = skk_session_buf (s);
 	if (skk_buffer_can_clear (buf)) {
 		skk_buffer_clear (buf);
-		skk_buffer_set_status (buf, SKK_JISX0208_LATIN_MODE);
+		skk_buffer_set_mode (buf, SKK_JISX0208_LATIN_MODE);
 	}
 	return;
 }
@@ -1330,6 +1497,25 @@ static void
 skk_process_keyevent_jmode (iml_session_t *s, IMKeyListEvent *kev)
 {
 	SkkBuffer *buf = skk_session_buf (s);
+	DEBUG_DO (printf ("skk_process_keyevent_jmode\n"));
+	DEBUG_DO (g_message ("skkbuf result (%s)\n", skkbuf_r (buf)));
+	DEBUG_DO (g_message ("skkbuf preedit (%s)\n", skkbuf_p (buf)));
+	DEBUG_DO (g_message ("skkbuf direction_word (%s)\n", skkbuf_d (buf)));
+	DEBUG_DO (g_message ("skkbuf in_buf (%s)\n", skkbuf_b (buf)));
+	#if 0
+	if (skk_buffer_can_commit (buf)) {
+		skk_commit_string (s, skkbuf_r (buf));
+		/* once commit */
+	}
+	if (skk_buffer_has_preedit (buf)) {
+		gchar *preedit = skk_buffer_get_preedit_string (buf);
+		skk_preedit_clear (s);
+		skk_preedit_draw (s,preedit);
+		g_free (preedit);
+	} else {
+		skk_preedit_clear (s);
+	}
+	#endif
 	if (skk_buffer_can_clear (buf)) {
 		skk_buffer_clear (buf);
 	}
@@ -1374,7 +1560,7 @@ skk_process_keyevent (iml_session_t * s, IMKeyListEvent * kev)
 	if (!skk_status) {
 		skk_status = SKK_NONE_MASK;
 	}
-	status = skk_keymap_do_func (buf, d->skkfunc, ch, skk_status);
+	status = skk_function_do_function (d->skkfunc, buf, ch, skk_status);
 	switch (status) {
 		case SKK_NOTHING:
 			return;
@@ -1386,7 +1572,8 @@ skk_process_keyevent (iml_session_t * s, IMKeyListEvent * kev)
 		case SKK_TRANS:
 			break;
 	}
-	switch (skk_buffer_get_status (buf)) {
+#if 0
+	switch (skk_buffer_get_mode (buf)) {
 		case SKK_J_MODE:
 			skk_process_keyevent_jmode (s,kev);
 			break;
@@ -1403,6 +1590,7 @@ skk_process_keyevent (iml_session_t * s, IMKeyListEvent * kev)
 			break;
 	}
 	return;
+#endif
 }
 
 /* FEEDBACK */
@@ -1735,6 +1923,8 @@ skk_preedit_draw (iml_session_t *s, unsigned char *str)
 		strs[0] = str;
 		strs[1] = str + cursor->revPos;
 		strs[2] = str + cursor->revPos + cursor->revLen;
+		g_message ("%s %s %s %d %d %d", strs[0], strs[1], strs[2],
+				nb[0], nb[1], nb[2]);
 		if (nb[2] == 0) {
 			p = skk_strtoimtext (s, 2, nb, strs, fts);
 		} else {
@@ -1755,6 +1945,8 @@ skk_preedit_draw (iml_session_t *s, unsigned char *str)
 		strs[0] = str;
 		strs[1] = str + strlen (str) - cur;
 		strs[2] = str + nb[0] +  nb[1];
+		g_message ("caret %s %s %s %d %d %d", strs[0], strs[1], strs[2],
+				nb[0], nb[1], nb[2]);
 		if (nb[2] == 0) {
 			p = skk_strtoimtext (s, 2, nb, strs, fts);
 		} else {
@@ -1769,8 +1961,10 @@ skk_preedit_draw (iml_session_t *s, unsigned char *str)
 		strs[0] = str;
 		p = skk_strtoimtext (s, 1, nb, strs, fts);
 	}
+#if 0
 	lp = s->If->m->iml_make_preedit_erase_inst (s);
 	s->If->m->iml_execute (s,&lp);
+#endif
 	lp = s->If->m->iml_make_preedit_draw_inst (s, p);
 	s->If->m->iml_execute (s, &lp);
 #if 0
@@ -1788,6 +1982,7 @@ skk_preedit_done (iml_session_t *s)
 	SkkSessionData *ses = skk_session_data (s);
 	if (ses->preedit_start == True) {
 		iml_inst *lp;
+		g_message ("preedit done");
 		lp = s->If->m->iml_make_preedit_done_inst (s);
 		s->If->m->iml_execute (s, &lp);
 		ses->preedit_start = False;
@@ -1800,8 +1995,11 @@ skk_preedit_clear (iml_session_t *s)
 	SkkSessionData *ses = skk_session_data (s);
 	if (ses->preedit_start == True) {
 		iml_inst *lp;
+		g_message ("preedit_erase_inst");
+#if 1
 		lp = s->If->m->iml_make_preedit_erase_inst (s);
 		s->If->m->iml_execute (s,&lp);
+#endif
 #if 0
 		lp = s->If->m->iml_make_preedit_done_inst (s);
 		s->If->m->iml_execute (s,&lp);
@@ -1858,7 +2056,7 @@ skk_aux_status_draw (iml_session_t *s)
 	UTFCHAR *send_string[1];
 	SkkSessionData *ses = skk_session_data (s);
 	SkkBuffer *buf = skk_session_buf (s);
-	SkkStatus status = skk_buffer_get_status (buf);
+	SkkMajorMode status = skk_buffer_get_mode (buf);
 
 	if (ses->palette_aux_ok == False)  {
 		skk_aux_start (s, SKK_AUX_PALETTE_CLASS_NAME);
@@ -1893,7 +2091,7 @@ skk_aux_status_draw (iml_session_t *s)
 		default:
 			break;
 	}
-	str = (gchar*) skk_buffer_get_status_string (buf);
+	str = (gchar*) skk_buffer_get_mode_string (buf);
 	ustr = skk_strtoutf (str);
 	send_string[0] = ustr;
 	skk_aux_draw (s, SKK_AUX_PALETTE_CLASS_NAME, 1, send_status, 1, send_string);
@@ -1940,7 +2138,7 @@ skk_status_draw (iml_session_t *s)
 		skk_status_draw_off (s);
 		return;
 	}
-	str = (gchar*)skk_buffer_get_status_string (buf);
+	str = (gchar*)skk_buffer_get_mode_string (buf);
 	ustr = skk_strtoutf (str);
 	p = skk_utftoimtext (s,ustr);
 	if (ses->status_start == False) {
@@ -2174,7 +2372,7 @@ skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 				break;
 			case SKK_MODE_LATIN:
 				skk_buffer_clear (buf);
-				skk_buffer_set_status (buf, SKK_LATIN_MODE);
+				skk_buffer_set_mode (buf, SKK_LATIN_MODE);
 				break;
 			case SKK_MODE_JISX0201_KANA:
 				skk_buffer_clear (buf);
@@ -2182,11 +2380,11 @@ skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 				break;
 			case SKK_MODE_JISX0208_LATIN:
 				skk_buffer_clear (buf);
-				skk_buffer_set_status (buf, SKK_JISX0208_LATIN_MODE);
+				skk_buffer_set_mode (buf, SKK_JISX0208_LATIN_MODE);
 				break;
 			case SKK_MODE_ABBREV:
 				skk_buffer_clear (buf);
-				skk_buffer_set_status (buf, SKK_ABBREV_MODE);
+				skk_buffer_set_mode (buf, SKK_ABBREV_MODE);
 				break;
 			case SKK_AUX_CODE_DEFINED:
 				{
@@ -2235,6 +2433,8 @@ skk_process_auxevent (iml_session_t *s, IMAuxDrawCallbackStruct *aux)
 				/* famao */
 
 				if (d->drop_priv) {
+					/* reset function map */
+					skk_function_map_clear (d->skkfuncmap);
 					/* reset conv_rule */
 					skk_conv_rule_clear (d->skkconv);
 					/* skk_conf_clear (d->skkconf); */
@@ -2311,27 +2511,13 @@ skk_status_done (iml_session_t *s)
 static Bool
 skk_drop_privilege (const char *username)
 {
-
-#ifndef NSS_BUFLEN_PASSWD
-#define NSS_BUELEN_PASSWD 1024
-#endif
-
-#ifdef HAVE_GETPWNAM_R
-	struct passwd pw_buf;
-	char buf[NSS_BUFLEN_PASSWD];
-#endif
 	struct passwd *pw;
 	uid_t uid;
 	gid_t gid;
 	if (!username)
 		goto error;
-#ifdef HAVE_GETPWNAM_R
-	if ((getpwnam_r (username, &pw_buf, buf, NSS_BUFLEN_PASSWD, &pw) != 0)) {
-		goto error;
-#else
 	if ((pw = getpwnam (username)) == NULL) {
 		goto error;
-#endif
 	} else {
 		uid = pw->pw_uid;
 		gid = pw->pw_gid;
@@ -2353,11 +2539,7 @@ skk_drop_privilege (const char *username)
 	return True;
 
 error:
-#ifdef HAVE_GETPWNAM_R
-	if ((getpwnam_r ("nobody", &pw_buf, buf, NSS_BUFLEN_PASSWD, &pw) == 0)) {
-#else
 	if ((pw = getpwnam ("nobody")) != NULL) {
-#endif
 		gid = pw->pw_gid;
 		uid = pw->pw_uid;
 		if (setregid (gid, gid) < 0) {
